# test_CardSetListModel.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Tests the Card List Model"""

from sutekh.tests.TestCore import SutekhTest
from sutekh.gui.CardListModel import CardListModelListener
from sutekh.gui.CardSetListModel import CardSetCardListModel, \
        NO_SECOND_LEVEL, SHOW_EXPANSIONS, SHOW_CARD_SETS, \
        EXPANSIONS_AND_CARD_SETS, CARD_SETS_AND_EXPANSIONS, \
        THIS_SET_ONLY, ALL_CARDS, PARENT_CARDS, CHILD_CARDS, \
        IGNORE_PARENT, PARENT_COUNT, MINUS_THIS_SET, MINUS_SETS_IN_USE
from sutekh.core import Filters, Groupings
from sutekh.core.SutekhObjects import PhysicalCardSet, IPhysicalCard, \
        IExpansion, IAbstractCard, MapPhysicalCardToPhysicalCardSet
# Needed to reduce speed impact of Grouping tests
from sutekh.core.SutekhObjectCache import SutekhObjectCache
from sutekh.core.DBSignals import send_changed_signal
import unittest

class CardSetListener(CardListModelListener):
    """Listener used in the test cases."""
    # pylint: disable-msg=W0231
    # CardListModelListener has no __init__
    def __init__(self):
        self.bLoadCalled = False
        self.aCards = []

    def load(self, aAbsCards):
        """Called when the model is loaded."""
        self.bLoadCalled = True
        self.aCards = aAbsCards

class CardSetListModelTests(SutekhTest):
    """Class for the test cases"""

    aParentCountToStr = ['IGNORE_PARENT', 'PARENT_COUNT', 'MINUS_THIS_SET',
            'MINUS_SETS_IN_USE']
    aExtraLevelToStr = ['NO_SECOND_LEVEL', 'SHOW_EXPANSIONS', 'SHOW_CARD_SETS',
            'EXPANSIONS_AND_CARD_SETS', 'CARD_SETS_AND_EXPANSIONS']
    aCardCountToStr = ['THIS_SET_ONLY', 'ALL_CARDS', 'PARENT_CARDS',
            'CHILD_CARDS']

    aNames = ['Test 1', 'Test Child 1', 'Test Grand Child', 'Test Sibling',
            'Test Grand Child 2']


    # pylint: disable-msg=R0201
    # I prefer to have these as methods
    def _count_second_level(self, oModel):
        """Count all the second level entries in the model."""
        iTotal = 0
        oIter = oModel.get_iter_first()
        while oIter:
            oChildIter = oModel.iter_children(oIter)
            while oChildIter:
                iTotal += oModel.iter_n_children(oChildIter)
                oChildIter = oModel.iter_next(oChildIter)
            oIter = oModel.iter_next(oIter)
        return iTotal

    def _count_all_cards(self, oModel):
        """Count all the entries in the model."""
        iTotal = 0
        oIter = oModel.get_iter_first()
        while oIter:
            iTotal += oModel.iter_n_children(oIter)
            oIter = oModel.iter_next(oIter)
        return iTotal

    def _get_all_child_counts(self, oModel, oIter):
        """Recursively descend the children of oIter, getting all the
           relevant info."""
        aList = []
        oChildIter = oModel.iter_children(oIter)
        while oChildIter:
            aList.append((oModel.get_card_count_from_iter(oChildIter),
                oModel.get_parent_count_from_iter(oChildIter),
                oModel.get_name_from_iter(oChildIter)))
            if oModel.iter_n_children(oChildIter) > 0:
                aList.extend(self._get_all_child_counts(oModel, oChildIter))
            oChildIter = oModel.iter_next(oChildIter)
        aList.sort()
        return aList

    def _get_all_counts(self, oModel):
        """Return a list of iCnt, iParCnt, sCardName tuples from the Model"""
        return self._get_all_child_counts(oModel, None)

    def _gen_card(self, sName, sExp):
        """Create a card given the name and Expansion"""
        if sExp:
            oExp = IExpansion(sExp)
        else:
            oExp = None
        oAbs = IAbstractCard(sName)
        return IPhysicalCard((oAbs, oExp))

    def _reset_modes(self, oModel):
        """Set the model to the minimal state."""
        # The database signal handling means that all CardSetListModels
        # associated with a card set will update when send_changed_signal is
        # called, so we reset the model state so these calls will be cheap if
        # this models is affected when we're not explicitly testing it.
        oModel.iParentCountMode = IGNORE_PARENT
        oModel.iLevelMode = NO_SECOND_LEVEL
        oModel.bEditable = False
        oModel.iShowMode = THIS_SET_ONLY

    # pylint: disable-msg=R0913
    # Need all these arguments here
    def _format_error(self, sErrType, oTest1, oTest2, oModel, oPCS=None):
        """Format an informative error message"""
        # pylint: disable-msg=W0212
        # Need info from _oCardSet here
        if oPCS:
            sModel = "Changing card set %s\n" % oPCS.name
        else:
            sModel = ''
        sModel += "Model: [card set %s, inuse=%s, groupby=%s]\n" % (
                    oModel._oCardSet.name, oModel._oCardSet.inuse,
                    oModel.groupby)
        sModel += " State : (ExtraLevelsMode %s, ParentCountMode : %s, " \
                "ShowCardMode : %s, Editable: %s)" % (
                        self.aExtraLevelToStr[oModel.iExtraLevelsMode],
                        self.aParentCountToStr[oModel.iParentCountMode],
                        self.aCardCountToStr[oModel.iShowCardMode],
                        oModel.bEditable)
        return "%s : %s vs %s\n%s" % (sErrType, oTest1, oTest2, sModel)

    # pylint: enable-msg=R0201
    # pylint: enable-msg=R0913

    def _add_remove_cards(self, oPCS, oModel, aPhysCards):
        """Helper function to add and remove distinct cards from the card set,
           validating that the model works correctly"""
        # pylint: disable-msg=E1101
        # SQLObjext confuses pylint
        oModel.sEditColour = "red"
        oModel.load()
        tStartTotals = (
                oModel.iter_n_children(None),
                self._count_all_cards(oModel),
                self._count_second_level(oModel))
        aStartList = self._get_all_counts(oModel)
        for oCard in aPhysCards:
            oPCS.addPhysicalCard(oCard.id)
            oPCS.syncUpdate()
            send_changed_signal(oPCS, oCard, 1)
        tAddTotals = (
                oModel.iter_n_children(None),
                self._count_all_cards(oModel),
                self._count_second_level(oModel))
        aAddList = self._get_all_counts(oModel)
        oModel.load()
        tLoadTotals = (
                oModel.iter_n_children(None),
                self._count_all_cards(oModel),
                self._count_second_level(oModel))
        aLoadList = self._get_all_counts(oModel)
        self.assertEqual(tAddTotals, tLoadTotals, self._format_error(
            "Totals for inc_card and load differ", tAddTotals, tLoadTotals,
            oModel, oPCS))
        self.assertEqual(aAddList, aLoadList, self._format_error(
            "Card Lists for inc_card and load differ", aAddList, aLoadList,
            oModel, oPCS))
        # Card removal
        # We use the map table, so we can also test dec_card properly
        for oCard in aPhysCards:
            oMapEntry = list(MapPhysicalCardToPhysicalCardSet.selectBy(
                    physicalCardID=oCard.id, physicalCardSetID=oPCS.id))[-1]
            MapPhysicalCardToPhysicalCardSet.delete(oMapEntry.id)
            oPCS.syncUpdate()
            send_changed_signal(oPCS, oCard, -1)
        tDecTotals = (
                oModel.iter_n_children(None),
                self._count_all_cards(oModel),
                self._count_second_level(oModel))
        aDecList = self._get_all_counts(oModel)
        # test that we've behaved sanely
        self.assertEqual(tDecTotals, tStartTotals, self._format_error(
            "Totals for dec_card and load differ", tDecTotals, tStartTotals,
            oModel, oPCS))
        self.assertEqual(aDecList, aStartList, self._format_error(
            "Card lists for dec_card and load differ", aDecList, aStartList,
            oModel, oPCS))

    def _loop_modes(self, oPCS, oModel):
        """Loop over all the possible modes of the model, calling
           _add_remove_cards to test the model."""
        aCards = [('AK-47', None), ('Bronwen', 'SW'), ('Cesewayo', None),
                ('Anna "Dictatrix11" Suljic', 'NoR'), ('Ablative Skin',
                    'Sabbat')] + [('Alexandra', 'CE'), ('Alexandra', None),
                ('Ablative Skin', None)] * 5
        aPhysCards = []
        for sName, sExp in aCards:
            oCard = self._gen_card(sName, sExp)
            aPhysCards.append(oCard)
        for bEditFlag in [False, True]:
            oModel.bEditable = bEditFlag
            for iLevelMode in [NO_SECOND_LEVEL, SHOW_EXPANSIONS,
                    SHOW_CARD_SETS, EXPANSIONS_AND_CARD_SETS,
                    CARD_SETS_AND_EXPANSIONS]:
                oModel.iExtraLevelsMode = iLevelMode
                for iParentMode in [IGNORE_PARENT, PARENT_COUNT,
                        MINUS_THIS_SET, MINUS_SETS_IN_USE]:
                    oModel.iParentCountMode = iParentMode
                    for iShowMode in [THIS_SET_ONLY, ALL_CARDS, PARENT_CARDS,
                            CHILD_CARDS]:
                        oModel.iShowCardMode = iShowMode
                        self._add_remove_cards(oPCS, oModel, aPhysCards)
        self._reset_modes(oModel)

    def _loop_zero_filter_modes(self, oModel):
        """Loop over all the possible modes of the model, calling
           a zero result filters to test the model."""
        for bEditFlag in [False, True]:
            oModel.bEditable = bEditFlag
            for iLevelMode in [NO_SECOND_LEVEL, SHOW_EXPANSIONS,
                    SHOW_CARD_SETS, EXPANSIONS_AND_CARD_SETS,
                    CARD_SETS_AND_EXPANSIONS]:
                oModel.iExtraLevelsMode = iLevelMode
                for iParentMode in [IGNORE_PARENT, PARENT_COUNT,
                        MINUS_THIS_SET, MINUS_SETS_IN_USE]:
                    oModel.iParentCountMode = iParentMode
                    for iShowMode in [THIS_SET_ONLY, ALL_CARDS, PARENT_CARDS,
                            CHILD_CARDS]:
                        oModel.iShowCardMode = iShowMode
                        oModel.selectfilter = Filters.CardNameFilter('ZZZZZZZ')
                        oModel.applyfilter = True
                        oModel.load()
                        tFilterTotals = (
                                oModel.iter_n_children(None),
                                self._count_all_cards(oModel),
                                self._count_second_level(oModel))
                        self.assertEqual(tFilterTotals, (1, 0, 0),
                                self._format_error("Totals for filter differ"
                                    " from expected zero result",
                                    tFilterTotals, (1, 0, 0), oModel))

    def test_basic(self):
        """Set of simple tests of the Card Set List Model"""
        # pylint: disable-msg=R0915, R0914
        # R0915, R0914: Want a long, sequential test case to minimise
        # repeated setups, so it has lots of lines + variables
        # pylint: disable-msg=W0612
        # oCache exists only for the internal cache, so it's unused
        oCache = SutekhObjectCache()
        # pylint: enable-msg=W0612
        oPCS = PhysicalCardSet(name=self.aNames[0])
        oModel = CardSetCardListModel(self.aNames[0])
        oListener = CardSetListener()
        oModel.load()
        self.assertFalse(oListener.bLoadCalled)
        oModel.add_listener(oListener)
        oModel.load()
        self.assertTrue(oListener.bLoadCalled)
        self.assertEquals(len(oListener.aCards), 0)
        # Check for the 'No cards' entry in the model
        self.assertEquals(oModel.iter_n_children(None), 1)
        aCards = [('Alexandra', 'CE'), ('Sha-Ennu', 'Third Edition')]
        for sName, sExp in aCards:
            oCard = self._gen_card(sName, sExp)
            # pylint: disable-msg=E1101
            # PyProtocols confuses pylint
            oPCS.addPhysicalCard(oCard.id)
        oModel.load()
        self.assertEqual(len(oListener.aCards), 2)
        # Only Vampires added
        self.assertEqual(oModel.iter_n_children(None), 1)
        oModel.groupby = Groupings.NullGrouping
        self.assertEqual(self._count_all_cards(oModel), 2)
        self.assertEqual(self._count_second_level(oModel), 2)
        # Check the drag-n-drop helper
        self.assertEqual(oModel.get_drag_child_info('0'), {})
        self.assertEqual(oModel.get_drag_child_info('0:0:0'), {})
        self.assertEqual(oModel.get_drag_child_info('0:0'),
                {'Camarilla Edition' : 1})
        self.assertEqual(oModel.get_drag_info_from_path('0:0:0'),
                (u"Alexandra", "Camarilla Edition", 1, 2))
        self.assertEqual(oModel.get_drag_info_from_path('0:0'),
                (u"Alexandra", None, 1, 1))
        self.assertEqual(oModel.get_drag_info_from_path('0'),
                (None, None, 2, 0))
        oModel.iExtraLevelsMode = NO_SECOND_LEVEL
        oModel.load()
        # This should also work for no expansions shown
        self.assertEqual(self._count_all_cards(oModel), 2)
        self.assertEqual(self._count_second_level(oModel), 0)
        self.assertEqual(oModel.get_drag_child_info('0'), {})
        self.assertEqual(oModel.get_drag_child_info('0:0'),
                {'Camarilla Edition' : 1})
        # Add Cards
        self._reset_modes(oModel)
        self._loop_modes(oPCS, oModel)
        # Check over all the groupings
        for cGrouping in [Groupings.CryptLibraryGrouping,
                Groupings.DisciplineGrouping, Groupings.ClanGrouping,
                Groupings.CardTypeGrouping, Groupings.ExpansionGrouping,
                Groupings.RarityGrouping]:
            oModel.groupby = cGrouping
            self._loop_modes(oPCS, oModel)

    def test_relationships(self):
        """Tests Model against more complex Card Set relationships"""
        # pylint: disable-msg=R0915, R0914
        # R0915, R0914: Want a long, sequential test case to minimise
        # repeated setups, so it has lots of lines + variables
        # pylint: disable-msg=W0612
        # oCache exists only for the internal cache, so it's unused
        oCache = SutekhObjectCache()
        # pylint: enable-msg=W0612
        oPCS = PhysicalCardSet(name=self.aNames[0])
        oModel = CardSetCardListModel(self.aNames[0])
        # Add cards
        aCards = [('Alexandra', 'CE'), ('Sha-Ennu', 'Third Edition'),
                ('Alexandra', 'CE'), ('Sha-Ennu', 'Third Edition'),
                ('Alexandra', None), ('Bronwen', 'Sabbat'),
                ('.44 Magnum', 'Jyhad'), ('.44 Magnum', 'Jyhad'),
                ('Yvette, The Hopeless', 'CE'),
                ('Yvette, The Hopeless', 'BSC')]
        for sName, sExp in aCards:
            oCard = self._gen_card(sName, sExp)
            # pylint: disable-msg=E1101
            # PyProtocols confuses pylint
            oPCS.addPhysicalCard(oCard.id)
        # Create a child card set with some entries and check everything works
        oChildPCS = PhysicalCardSet(name=self.aNames[1], parent=oPCS)
        for sName, sExp in aCards[2:6]:
            oCard = self._gen_card(sName, sExp)
            # pylint: disable-msg=E1101
            # PyProtocols confuses pylint
            oChildPCS.addPhysicalCard(oCard.id)
        oChildPCS.inuse = False
        # Check adding cards when we have a parent card set
        self._loop_modes(oPCS, oModel)
        # Change model to other card set
        oModel.update_to_new_db(self.aNames[1])
        self._loop_modes(oChildPCS, oModel)
        # Check adding cards when we have a child, but no parent
        # And when we're in use
        oChildPCS.inuse = True
        self._loop_modes(oChildPCS, oModel)
        # change back
        oModel.update_to_new_db(self.aNames[0])
        self._loop_modes(oPCS, oModel)
        # Add a grand child
        oGrandChildPCS = PhysicalCardSet(name=self.aNames[2], parent=oChildPCS)
        for sName, sExp in aCards[3:7]:
            oCard = self._gen_card(sName, sExp)
            # pylint: disable-msg=E1101
            # PyProtocols confuses pylint
            oGrandChildPCS.addPhysicalCard(oCard.id)
        oGrandChildPCS.inuse = False
        # Check adding cards when we have a parent card set and a child
        oModel.update_to_new_db(self.aNames[1])
        self._loop_modes(oChildPCS, oModel)
        oGrandChildPCS.inuse = True
        self._loop_modes(oChildPCS, oModel)
        # Add some cards to oGrandChildPCS that aren't in parent and oChildPCS,
        # add a sibling card set to oChildPCS and add another child and retest
        oSibPCS = PhysicalCardSet(name=self.aNames[3], parent=oPCS)
        for sName, sExp in aCards[1:6]:
            oCard = self._gen_card(sName, sExp)
            # pylint: disable-msg=E1101
            # PyProtocols confuses pylint
            oSibPCS.addPhysicalCard(oCard.id)
        oSibPCS.inuse = True
        oGrandChild2PCS = PhysicalCardSet(name=self.aNames[4],
                parent=oChildPCS)
        oGrandChild2PCS.inuse = True
        aCards = [('AK-47', 'LotN'), ('Cesewayo', 'LoB'),
                ('Aire of Elation', 'CE'), ('Yvette, The Hopeless', None),
                ('Yvette, The Hopeless', 'BSC')]
        for sName, sExp in aCards:
            oCard = self._gen_card(sName, sExp)
            # pylint: disable-msg=E1101
            # PyProtocols confuses pylint
            oGrandChild2PCS.addPhysicalCard(oCard.id)
            if sName == 'Aire of Elation':
                oGrandChildPCS.addPhysicalCard(oCard.id)
                oGrandChildPCS.addPhysicalCard(oCard.id)
                oGrandChildPCS.addPhysicalCard(oCard.id)
                oGrandChildPCS.syncUpdate()
            oGrandChild2PCS.syncUpdate()
        oGrandChildPCS.inuse = False
        self._loop_modes(oChildPCS, oModel)
        oGrandChildPCS.inuse = True
        self._loop_modes(oChildPCS, oModel)
        # pylint: disable-msg=E1101
        # PyProtocols confuses pylint
        oGrandChild2PCS.addPhysicalCard(self._gen_card('Ablative Skin',
            'Sabbat'))
        self._loop_modes(oChildPCS, oModel)
        oChildPCS.addPhysicalCard(self._gen_card('Ablative Skin',
            'Sabbat'))
        oGrandChild2PCS.addPhysicalCard(self._gen_card(
            'Ablative Skin', None))
        self._loop_modes(oChildPCS, oModel)
        # Point model at grandchild
        oModel.update_to_new_db(self.aNames[2])
        # Test adding cards to a sibling card set
        self._loop_modes(oGrandChild2PCS, oModel)
        # point at child
        oModel.update_to_new_db(self.aNames[1])
        self._loop_modes(oSibPCS, oModel)
        # Test adding cards to the parent card set
        self._loop_modes(oPCS, oModel)
        # point at grand child
        oModel.update_to_new_db(self.aNames[2])
        self._loop_modes(oChildPCS, oModel)
        # Test adding cards to the child card set - point at parent
        oModel.update_to_new_db(self.aNames[0])
        self._loop_modes(oChildPCS, oModel)
        self._loop_modes(oSibPCS, oModel)
        # point at child
        oModel.update_to_new_db(self.aNames[1])
        self._loop_modes(oGrandChildPCS, oModel)
        # Go through some of grouping tests as well
        # We want to ensure that this works with non-NullGroupings,
        # but we don't need to cover all the groupings again
        for cGrouping in [Groupings.DisciplineGrouping,
                Groupings.CardTypeGrouping]:
            oModel.groupby = cGrouping
            self._loop_modes(oSibPCS, oModel)
            self._loop_modes(oPCS, oModel)
            self._loop_modes(oGrandChildPCS, oModel)

    def test_filters(self):
        """Test filtering for the card set"""
        # pylint: disable-msg=R0915
        # R0915: Want a long, sequential test case to reduce
        # repeated setups, so it has lots of lines
        # pylint: disable-msg=W0612
        # oCache exists only for the internal cache, so it's unused
        oCache = SutekhObjectCache()
        # pylint: enable-msg=W0612
        oPCS = PhysicalCardSet(name=self.aNames[0])
        oModel = CardSetCardListModel(self.aNames[0])
        aCards = [('Alexandra', 'CE'), ('Sha-Ennu', 'Third Edition'),
                ('Alexandra', None), ('Bronwen', 'Sabbat'),
                ('.44 Magnum', 'Jyhad'), ('.44 Magnum', 'Jyhad'),
                ('Yvette, The Hopeless', 'CE'),
                ('Yvette, The Hopeless', 'BSC')]
        for sName, sExp in aCards:
            oCard = self._gen_card(sName, sExp)
            # pylint: disable-msg=E1101
            # PyProtocols confuses pylint
            oPCS.addPhysicalCard(oCard.id)
        # Test filter which selects nothing works
        self._loop_zero_filter_modes(oModel)
        # Check basic filtering
        oModel.iShowCardMode = THIS_SET_ONLY
        oModel.iParentCountMode = IGNORE_PARENT
        oModel.iExtraLevelsMode = NO_SECOND_LEVEL
        oModel.bEditable = False
        oModel.groupby = Groupings.NullGrouping
        # Test card type
        oModel.selectfilter = Filters.CardTypeFilter('Vampire')
        oModel.applyfilter = True
        oModel.load()
        tTotals = (oModel.iter_n_children(None),
                self._count_all_cards(oModel),
                self._count_second_level(oModel))
        tExpected = (1, 4, 0)
        self.assertEqual(tTotals, tExpected, 'Wrong results from filter : '
                '%s vs %s' % (tTotals, tExpected))
        oModel.groupby = Groupings.DisciplineGrouping
        oModel.applyfilter = True
        oModel.load()
        tTotals = (oModel.iter_n_children(None),
                self._count_all_cards(oModel),
                self._count_second_level(oModel))
        tExpected = (11, 18, 0)
        self.assertEqual(tTotals, tExpected, 'Wrong results from filter : '
                '%s vs %s' % (tTotals, tExpected))
        oModel.iExtraLevelsMode = SHOW_EXPANSIONS
        oModel.load()
        tTotals = (oModel.iter_n_children(None),
                self._count_all_cards(oModel),
                self._count_second_level(oModel))
        tExpected = (11, 18, 25)
        self.assertEqual(tTotals, tExpected, 'Wrong results from filter : '
                '%s vs %s' % (tTotals, tExpected))
        oModel.bEditable = True
        oModel.load()
        tTotals = (oModel.iter_n_children(None),
                self._count_all_cards(oModel),
                self._count_second_level(oModel))
        tExpected = (11, 18, 48)
        self.assertEqual(tTotals, tExpected, 'Wrong results from filter : '
                '%s vs %s' % (tTotals, tExpected))
        # Add a child card set, and test filtering results
        oModel.groupby = Groupings.NullGrouping
        oChildPCS = PhysicalCardSet(name=self.aNames[1], parent=oPCS,
                inuse=True)
        aCards = [('Sha-Ennu', None),
                ('Kabede Maru', None),
                ('Gracis Nostinus', 'CE'),
                ('Yvette, The Hopeless', 'BSC')]
        for sName, sExp in aCards:
            oCard = self._gen_card(sName, sExp)
            # pylint: disable-msg=E1101
            # PyProtocols confuses pylint
            oChildPCS.addPhysicalCard(oCard.id)
        oModel.bEditable = False
        oModel.load()
        tTotals = (oModel.iter_n_children(None),
                self._count_all_cards(oModel),
                self._count_second_level(oModel))
        tExpected = (1, 4, 6)
        self.assertEqual(tTotals, tExpected, 'Wrong results from filter : '
                '%s vs %s' % (tTotals, tExpected))
        oModel.iExtraLevelsMode = SHOW_CARD_SETS
        oModel.load()
        tTotals = (oModel.iter_n_children(None),
                self._count_all_cards(oModel),
                self._count_second_level(oModel))
        tExpected = (1, 4, 2)
        self.assertEqual(tTotals, tExpected, 'Wrong results from filter : '
                '%s vs %s' % (tTotals, tExpected))
        oModel.iShowCardMode = CHILD_CARDS
        oModel.load()
        tTotals = (oModel.iter_n_children(None),
                self._count_all_cards(oModel),
                self._count_second_level(oModel))
        tExpected = (1, 6, 4)
        self.assertEqual(tTotals, tExpected, 'Wrong results from filter : '
                '%s vs %s' % (tTotals, tExpected))
        oModel.iShowCardMode = ALL_CARDS
        oModel.load()
        tTotals = (oModel.iter_n_children(None),
                self._count_all_cards(oModel),
                self._count_second_level(oModel))
        tExpected = (1, 26, 26)
        self.assertEqual(tTotals, tExpected, 'Wrong results from filter : '
                '%s vs %s' % (tTotals, tExpected))

    def test_empty(self):
        """Test corner cases around empty card sets"""
        # pylint: disable-msg=W0612
        # oCache exists only for the internal cache, so it's unused
        oCache = SutekhObjectCache()
        # pylint: enable-msg=W0612
        oPCS = PhysicalCardSet(name=self.aNames[0])
        oChildPCS = PhysicalCardSet(name=self.aNames[1], parent=oPCS)
        oGrandChildPCS = PhysicalCardSet(name=self.aNames[2], parent=oChildPCS)
        oChildModel = CardSetCardListModel(self.aNames[1])
        oChildModel.groupby = Groupings.NullGrouping
        self._loop_modes(oChildPCS, oChildModel)
        self._loop_modes(oPCS, oChildModel)
        self._loop_modes(oGrandChildPCS, oChildModel)
        # Add some cards to parent + child card sets
        aCards = [('Alexandra', 'CE'), ('Sha-Ennu', 'Third Edition'),
                ('Alexandra', None), ('Bronwen', 'Sabbat'),
                ('.44 Magnum', 'Jyhad'), ('.44 Magnum', 'Jyhad'),
                ('Yvette, The Hopeless', 'CE'),
                ('Yvette, The Hopeless', 'BSC')]
        for sName, sExp in aCards:
            oCard = self._gen_card(sName, sExp)
            # pylint: disable-msg=E1101
            # PyProtocols confuses pylint
            oPCS.addPhysicalCard(oCard.id)
            if sName != 'Yvette, The Hopeless':
                oGrandChildPCS.addPhysicalCard(oCard.id)
        self._loop_modes(oChildPCS, oChildModel)

if __name__ == "__main__":
    unittest.main()



# CardSetExportFELDB.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Plugin for exporting to ELDB's deck & inventory formats"""

import gtk
from sutekh.core.SutekhObjects import PhysicalCardSet
from sutekh.gui.PluginManager import CardListPlugin
from sutekh.gui.SutekhFileWidget import ExportDialog
from sutekh.io.WriteELDBInventory import WriteELDBInventory
from sutekh.io.WriteELDBDeckFile import WriteELDBDeckFile
from sutekh.SutekhUtility import safe_filename

class CardSetExportFELDB(CardListPlugin):
    """Provides a dialog for selecting a filename, then calls on
       WriteELDB* to produce the required output."""
    dTableVersions = { PhysicalCardSet: [5]}
    aModelsSupported = [PhysicalCardSet]

    def get_menu_item(self):
        """Register on the 'Export Card Set' Menu"""
        if not self.check_versions() or not self.check_model_type():
            return None
        oExport = gtk.MenuItem("Export to FELDB file")
        oExport.connect("activate", self.make_dialog)
        return ('Export Card Set', oExport)

    # pylint: disable-msg=W0613
    # oWidget has to be here, although it's unused
    def make_dialog(self, oWidget):
        """Create the dialog"""
        # pylint: disable-msg=E1101
        # vbox confuses pylint
        oDlg = ExportDialog("Choose FileName for Exported CardSet",
                self.parent, '%s_ELDB.eld' % safe_filename(self.view.sSetName))
        oDlg.add_filter_with_pattern('ELD Files', ['*.eld'])
        oDlg.add_filter_with_pattern('CSV Files', ['*.csv'])
        oFirstBut = gtk.RadioButton(None, 'ELDB Deck File', False)
        oDlg.vbox.pack_start(oFirstBut, expand=False)
        oSecondBut = gtk.RadioButton(oFirstBut, 'ELDB Inventory CSV File')
        oDlg.vbox.pack_start(oSecondBut, expand=False)
        oDlg.show_all()
        oDlg.run()
        self.handle_response(oDlg.get_name(), oFirstBut)

    # pylint: enable-msg=W0613

    def handle_response(self, sFileName, oFirstBut):
        """Handle the users response. Write the output to file."""
        # pylint: disable-msg=E1101
        # SQLObject methods confuse pylint
        if sFileName is not None:
            oCardSet = self.get_card_set()
            if not oCardSet:
                return
            fOut = file(sFileName,"w")
            if oFirstBut.get_active():
                oWriter = WriteELDBDeckFile()
                oWriter.write(fOut, oCardSet)
            else:
                oWriter = WriteELDBInventory()
                oWriter.write(fOut, oCardSet)
            fOut.close()

# pylint: disable-msg=C0103
# accept plugin name
plugin = CardSetExportFELDB



# Groupings.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2006 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Provide classes to change the way cards are grouped in the display"""

# Base Grouping Class

class IterGrouping(object):
    """Bass class for the groupings"""
    def __init__(self, oIter, fKeys):
        """Create the grouping

           oIter: Iterable to group.
           fKeys: Function which maps an item from the iterable
                  to a list of keys. Keys must be hashable.
           """
        self.__oIter = oIter
        self.__fKeys = fKeys

    def __iter__(self):
        dKeyItem = {}
        for oItem in self.__oIter:
            aSet = set(self.__fKeys(oItem))
            if len(aSet) == 0:
                dKeyItem.setdefault(None, []).append(oItem)
            else:
                for oKey in aSet:
                    dKeyItem.setdefault(oKey, []).append(oItem)

        aList = dKeyItem.keys()
        aList.sort()

        for oKey in aList:
            yield oKey, dKeyItem[oKey]

# Individual Groupings
#
# If you need to group PhysicalCards,
# set fGetCard to lambda x: x.abstractCard

# pylint: disable-msg=E0602
# pylint is confused by the lambda x: x construction
fDefGetCard = lambda x: x
# pylint: enable-msg=E0602

# pylint: disable-msg=C0111
# class names are pretty self-evident, so skip docstrings
class CardTypeGrouping(IterGrouping):
    def __init__(self, oIter, fGetCard=fDefGetCard):
        super(CardTypeGrouping, self).__init__(oIter,
                lambda x: [y.name for y in fGetCard(x).cardtype])

class MultiTypeGrouping(IterGrouping):
    """Group by card type, but make separate groupings for
       cards which have multiple types, e.g. Action Modifier / Reaction.
       """

    def __init__(self, oIter, fGetCard=fDefGetCard):
        # pylint: disable-msg=C0103
        # we accept x here for consistency with other groupings
        def multitype(x):
            """Return a list of one string with slash separated card types."""
            aTypes = [y.name for y in fGetCard(x).cardtype]
            aTypes.sort()
            return [" / ".join(aTypes)]
        super(MultiTypeGrouping, self).__init__(oIter, multitype)

class ClanGrouping(IterGrouping):
    """Group the cards by clan and/or creed"""
    def __init__(self, oIter, fGetCard=fDefGetCard):
        super(ClanGrouping, self).__init__(oIter, self._get_values)
        self.fGetCard = fGetCard

    def _get_values(self, oCard):
        """Get the values to group by for this card"""
        oThisCard = self.fGetCard(oCard)
        if oThisCard.creed:
            return [y.name for y in oThisCard.creed]
        else:
            return [y.name for y in oThisCard.clan]

class DisciplineGrouping(IterGrouping):
    """Group by Discipline or Virtue"""
    def __init__(self, oIter, fGetCard=fDefGetCard):
        super(DisciplineGrouping, self).__init__(oIter, self._get_values)
        self.fGetCard = fGetCard

    def _get_values(self, oCard):
        """Get the values to group by for this card"""
        oThisCard = self.fGetCard(oCard)
        if oThisCard.virtue:
            return [y.fullname for y in oThisCard.virtue]
        else:
            return [y.discipline.fullname for y in oThisCard.discipline]

class ExpansionGrouping(IterGrouping):
    def __init__(self, oIter, fGetCard=fDefGetCard):
        super(ExpansionGrouping, self).__init__(oIter,
                lambda x: [y.expansion.name for y in fGetCard(x).rarity])

class RarityGrouping(IterGrouping):
    def __init__(self, oIter, fGetCard=fDefGetCard):
        super(RarityGrouping, self).__init__(oIter,
                lambda x: [y.rarity.name for y in fGetCard(x).rarity])

class CryptLibraryGrouping(IterGrouping):
    def __init__(self, oIter, fGetCard=fDefGetCard):
        # Vampires and Imbued have exactly one card type (we hope that WW
        # don't change that)
        super(CryptLibraryGrouping, self).__init__(oIter,
                lambda x: [fGetCard(x).cardtype[0].name in ["Vampire",
                    "Imbued"] and "Crypt" or "Library"])

class SectGrouping(IterGrouping):
    """Group by Sect"""
    def __init__(self, oIter, fGetCard=fDefGetCard):
        super(SectGrouping, self).__init__(oIter,
                lambda x: [y.name for y in fGetCard(x).sect])

class TitleGrouping(IterGrouping):
    """Group by Title"""
    def __init__(self, oIter, fGetCard=fDefGetCard):
        super(TitleGrouping, self).__init__(oIter,
            lambda x: [y.name for y in fGetCard(x).title])

class CostGrouping(IterGrouping):
    """Group by Cost"""
    def __init__(self, oIter, fGetCard=fDefGetCard):
        super(CostGrouping, self).__init__(oIter, self._get_values)
        self.fGetCard = fGetCard

    def _get_values(self, oCard):
        """Get the values to group by for this card"""
        oThisCard = self.fGetCard(oCard)
        if oThisCard.cost:
            if oThisCard.cost == -1:
                return ['X %s' % oThisCard.costtype]
            else:
                return ['%d %s' % (oThisCard.cost, oThisCard.costtype)]
        else:
            return []

class NullGrouping(IterGrouping):
    # pylint: disable-msg=W0613
    # fGetCard is required by function signature
    def __init__(self, oIter, fGetCard=fDefGetCard):
        super(NullGrouping, self).__init__(oIter, lambda x: ["All"])



# ExtraCardViewColumns.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2007 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details
"""Disply extra columns in the tree view"""

import gtk
from sutekh.gui.PluginManager import CardListPlugin
from sutekh.gui.SutekhDialog import SutekhDialog
from sutekh.gui.CellRendererIcons import CellRendererIcons, SHOW_TEXT_ONLY, \
        SHOW_ICONS_ONLY, SHOW_ICONS_AND_TEXT
from sutekh.core.SutekhObjects import AbstractCard, PhysicalCard, \
           PhysicalCardSet
from sqlobject import SQLObjectNotFound

class ExtraCardViewColumns(CardListPlugin):
    """Add extra columns to the card list view.

       Allow the card list to be sorted on these columns
       """
    dTableVersions = {}
    aModelsSupported = [PhysicalCardSet, PhysicalCard]

    _dWidths = {
            'Card Type' : 100,
            'Clans and Creeds' : 100,
            'Disciplines and Virtues' : 150,
            'Expansions' : 600,
            'Group' : 40,
            'Capacity or Life(Imbued)' : 40,
            'Cost' : 100,
            }

    _dModes = {
            'Show Icons and Names' : SHOW_ICONS_AND_TEXT,
            'Show Icons only' : SHOW_ICONS_ONLY,
            'Show Text only' : SHOW_TEXT_ONLY,
            }

    # pylint: disable-msg=W0142
    # **magic OK here
    def __init__(self, *aArgs, **kwargs):
        super(ExtraCardViewColumns, self).__init__(*aArgs, **kwargs)
        self._dCols = {}
        self._dCols['Card Type'] = self._render_card_type
        self._dCols['Clans and Creeds'] = self._render_clan
        self._dCols['Disciplines and Virtues'] = self._render_disciplines
        self._dCols['Expansions'] = self._render_expansions
        self._dCols['Group'] = self._render_group
        self._dCols['Title'] = self._render_title
        self._dCols['Sect'] = self._render_sect
        self._dCols['Card Text'] = self._render_card_text
        self._dCols['Capacity or Life(Imbued)'] = self._render_capacity
        self._dCols['Cost'] = self._render_cost

        self._dSortDataFuncs = {}
        self._dSortDataFuncs['Card Type'] = self._get_data_cardtype
        self._dSortDataFuncs['Clans and Creeds'] = self._get_data_clan
        self._dSortDataFuncs['Disciplines and Virtues'] = \
                self._get_data_disciplines
        self._dSortDataFuncs['Expansions'] = self._get_data_expansions
        self._dSortDataFuncs['Group'] = self._get_data_group
        self._dSortDataFuncs['Title'] = self._get_data_title
        self._dSortDataFuncs['Sect'] = self._get_data_sect
        self._dSortDataFuncs['Card Text'] = self._get_data_card_text
        self._dSortDataFuncs['Capacity or Life(Imbued)'] = \
                self._get_data_capacity
        self._dSortDataFuncs['Cost'] = self._get_data_cost_sortkey

        self._dCardCache = {} # Used for sorting
        self._iShowMode = SHOW_ICONS_AND_TEXT
        self._oFirstBut = None
    # pylint: enable-msg=W0142

    # Rendering Functions

    def _get_card(self, oIter):
        """For the given iterator, get the associated abstract card"""
        if self.model.iter_depth(oIter) == 1:
            # Only try and lookup things that look like they should be cards
            try:
                sCardName = self.model.get_name_from_iter(oIter).lower()
                # Cache lookups, so we don't hit the database so hard when
                # sorting
                if not self._dCardCache.has_key(sCardName):
                    # pylint: disable-msg=E1101
                    # pylint + AbstractCard method wierdness
                    self._dCardCache[sCardName] = \
                            AbstractCard.byCanonicalName(sCardName)
                return self._dCardCache[sCardName]
            except SQLObjectNotFound:
                return None
        else:
            return None

    # pylint: disable-msg=R0201, W0613
    # Making these functions for clarity
    # several unused paramaters due to function signatures
    # The bGetIcons parameter is needed to avoid icon lookups, etc when
    # sorting
    def _get_data_cardtype(self, oCard, bGetIcons=True):
        """Return the card type"""
        if not oCard is None:
            aTypes = [x.name for x in oCard.cardtype]
            aTypes.sort()
            aIcons = []
            if bGetIcons:
                dIcons = self.icon_manager.get_icon_list(oCard.cardtype)
                if dIcons:
                    aIcons = [dIcons[x] for x in aTypes]
            return " /|".join(aTypes).split("|"), aIcons
        return [], []

    def _render_card_type(self, oColumn, oCell, oModel, oIter):
        """display the card type(s)"""
        oCard = self._get_card(oIter)
        aText, aIcons = self._get_data_cardtype(oCard, True)
        oCell.set_data(aText, aIcons, self._iShowMode)

    def _get_data_clan(self, oCard, bGetIcons=True):
        """get the clan for the card"""
        if not oCard is None:
            aClans = [x.name for x in oCard.clan]
            aIcons = []
            if aClans:
                aClans.sort()
                if bGetIcons:
                    dIcons = self.icon_manager.get_icon_list(oCard.clan)
                    if dIcons:
                        aIcons = [dIcons[x] for x in aClans]
                return " /|".join(aClans).split("|"), aIcons
            else:
                aCreed = [x.name for x in oCard.creed]
                aCreed.sort()
                if bGetIcons:
                    dIcons = self.icon_manager.get_icon_list(oCard.creed)
                    if dIcons:
                        aIcons = [dIcons[x] for x in aCreed]
                return " /|".join(aCreed).split("|"), aIcons
        return [], []

    def _render_clan(self, oColumn, oCell, oModel, oIter):
        """display the clan"""
        oCard = self._get_card(oIter)
        aText, aIcons = self._get_data_clan(oCard)
        oCell.set_data(aText, aIcons, self._iShowMode)

    def _get_data_disciplines(self, oCard, bGetIcons=True):
        """get disipline info for card"""
        if not oCard is None:
            # There are better ways of doing this, but this is 1st draft
            # attempt, so we'll do the firt that occurs to me
            aInfo = [((oP.level != 'superior') and oP.discipline.name or
                oP.discipline.name.upper(), oP.discipline.name) for
                oP in oCard.discipline]
            if aInfo:
                aInfo.sort(key=lambda x: x[0])
                if bGetIcons:
                    dIcons = self.icon_manager.get_icon_list(oCard.discipline)
                    aIcons = [dIcons[x[1]] for x in aInfo]
                else:
                    aIcons = []
                aDis = ", ".join([x[0] for x in aInfo]).split(" ")
                return aDis, aIcons
            else:
                aInfo = [oV.name for oV in oCard.virtue]
                if aInfo:
                    aInfo.sort()
                    if bGetIcons:
                        dIcons = self.icon_manager.get_icon_list(oCard.virtue)
                        aIcons = [dIcons[x] for x in aInfo]
                    else:
                        aIcons = []
                    aVirt = ", ".join(aInfo).split(" ")
                    return aVirt, aIcons
        return [], []

    def _render_disciplines(self, oColumn, oCell, oModel, oIter):
        """display the card disciplines"""
        oCard = self._get_card(oIter)
        aText, aIcons = self._get_data_disciplines(oCard)
        oCell.set_data(aText, aIcons, self._iShowMode)

    def _get_data_expansions(self, oCard, bGetIcons=True):
        """get expansion info"""
        if not oCard is None:
            aExp = [oP.expansion.shortname + "(" + oP.rarity.name + ")" for
                    oP in oCard.rarity]
            aExp.sort()
            aIcons = []
            if bGetIcons:
                aIcons = [None]*len(aExp)
            return aExp, aIcons
        return [], []

    def _render_expansions(self, oColumn, oCell, oModel, oIter):
        """Display expanson info"""
        oCard = self._get_card(oIter)
        aText, aIcons = self._get_data_expansions(oCard)
        oCell.set_data(aText, aIcons, self._iShowMode)

    def _get_data_group(self, oCard, bGetIcons=True):
        """get the group info for the card"""
        if not oCard is None and not oCard.group is None:
            return oCard.group, [None]
        # We use -1 for the any group, so flag with a very different number
        return -100, [None]

    def _render_group(self, oColumn, oCell, oModel, oIter):
        """Display the group info"""
        oCard = self._get_card(oIter)
        iGrp, aIcons = self._get_data_group(oCard)
        if iGrp != -100:
            if iGrp == -1:
                oCell.set_data(['Any'], aIcons, SHOW_TEXT_ONLY)
            else:
                oCell.set_data([str(iGrp)], aIcons, SHOW_TEXT_ONLY)
        else:
            oCell.set_data([""], aIcons, SHOW_TEXT_ONLY)

    def _get_data_capacity(self, oCard, bGetIcons=True):
        """Get the card's capacity"""
        if not oCard is None and not oCard.capacity is None:
            return oCard.capacity, [None]
        if not oCard is None and not oCard.life is None:
            return oCard.life, [None]
        return -1, [None]

    def _render_capacity(self, oColumn, oCell, oModel, oIter):
        """Display capacity in the column"""
        oCard = self._get_card(oIter)
        iCap, aIcons = self._get_data_capacity(oCard)
        if iCap != -1:
            oCell.set_data([str(iCap)], aIcons, SHOW_TEXT_ONLY)
        else:
            oCell.set_data([""], aIcons, SHOW_TEXT_ONLY)

    def _get_data_cost(self, oCard, bGetIcons=True):
        """Get the card's cost"""
        if not oCard is None and not oCard.cost is None:
            return oCard.cost, oCard.costtype, [None]
        return 0, "", [None]

    def _get_data_cost_sortkey(self, oCard, bGetIcons=True):
        """Get the sort key for sorting by cost.

           We want to group the cost types together, since the different
           types aren't comparable, hence the key is constructed as
           costtype + cost.
           We ensure that cost X cards sort after other values.
           """
        iCost, sCostType, aIcons = self._get_data_cost(oCard, bGetIcons)
        if iCost > 0:
            sKey = "%s %d" % (sCostType, iCost)
        elif iCost == -1:
            sKey = "%s X" % sCostType
        else:
            sKey = ""
        return sKey, aIcons

    def _render_cost(self, oColumn, oCell, oModel, oIter):
        """Display cost in the column"""
        oCard = self._get_card(oIter)
        iCost, sCostType, aIcons = self._get_data_cost(oCard)
        if iCost > 0:
            oCell.set_data(["%d %s" % (iCost, sCostType)], aIcons,
                SHOW_TEXT_ONLY)
        elif iCost == -1:
            oCell.set_data(["X %s" % sCostType], aIcons,
                SHOW_TEXT_ONLY)
        else:
            oCell.set_data([""], aIcons, SHOW_TEXT_ONLY)


    def _get_data_title(self, oCard, bGetIcons=True):
        """Get the card's title."""
        if not oCard is None:
            aTitles = [oT.name for oT in oCard.title]
            aTitles.sort()
            aIcons = []
            if bGetIcons:
                aIcons = [None]*len(aTitles)
            return aTitles, aIcons
        return [], []

    def _render_title(self, oColumn, oCell, oModel, oIter):
        """Display title in the column"""
        oCard = self._get_card(oIter)
        aTitles, aIcons = self._get_data_title(oCard)
        oCell.set_data(aTitles, aIcons, SHOW_TEXT_ONLY)

    def _get_data_sect(self, oCard, bGetIcons=True):
        """Get the card's sect."""
        if not oCard is None:
            aSects = [oS.name for oS in oCard.sect]
            aSects.sort()
            aIcons = []
            if bGetIcons:
                aIcons = [None]*len(aSects)
            return aSects, aIcons
        return [], []

    def _render_sect(self, oColumn, oCell, oModel, oIter):
        """Display sect in the column"""
        oCard = self._get_card(oIter)
        aSects, aIcons = self._get_data_sect(oCard)
        oCell.set_data(aSects, aIcons, SHOW_TEXT_ONLY)

    def _get_data_card_text(self, oCard, bGetIcons=True):
        """Get the card's card text."""
        if not oCard is None:
            aTexts = [oCard.text.replace("\n", " ")]
            aIcons = []
            if bGetIcons:
                aIcons = [None]*len(aTexts)
            return aTexts, aIcons
        return [], []

    def _render_card_text(self, oColumn, oCell, oModel, oIter):
        """Display card text in the column"""
        oCard = self._get_card(oIter)
        aTexts, aIcons = self._get_data_card_text(oCard)
        oCell.set_data(aTexts, aIcons, SHOW_TEXT_ONLY)

    # pylint: enable-msg=R0201
    # Dialog and Menu Item Creation

    def get_menu_item(self):
        """Register on 'Plugins' menu"""
        if not self.check_versions() or not self.check_model_type():
            return None
        oSelector = gtk.MenuItem("Select Extra Columns")
        oSelector.connect("activate", self.activate)
        return ('Plugins', oSelector)

    # W0613 - oWidget required by function signature
    def activate(self, oWidget):
        """Handle menu activation"""
        oDlg = self.make_dialog()
        oDlg.run()

    # W0613: oModel required by gtk's function signature
    def sort_column(self, oModel, oIter1, oIter2, oGetData):
        """Stringwise comparision of oIter1 and oIter2.

           Return -1 if oIter1 < oIter, 0 in ==, 1 if >
           """
        oCard1 = self._get_card(oIter1)
        oCard2 = self._get_card(oIter2)
        if oCard1 is None or oCard2 is None:
            # Not comparing cards, sort on name only
            sName1 = self.model.get_name_from_iter(oIter1).lower()
            sName2 = self.model.get_name_from_iter(oIter2).lower()
            if sName1 < sName2:
                iRes = -1
            elif sName1 > sName2:
                iRes = 1
            else:
                iRes = 0
            return iRes
        # Don't use icon info when sorting
        oVal1 = oGetData(oCard1, False)[0]
        oVal2 = oGetData(oCard2, False)[0]
        # convert to string for sorting
        if isinstance(oVal1, list):
            oVal1 = " ".join(oVal1)
            oVal2 = " ".join(oVal2)
        if oVal1 < oVal2:
            iRes = -1
        elif oVal1 > oVal2:
            iRes = 1
        else:
            # Values agrees, so sort on card name
            iRes = cmp(oCard1.name, oCard2.name)
        return iRes

    # pylint: enable-msg=W0613

    def make_dialog(self):
        """Create the column selection dialog"""
        sName = "Select Extra Columns ..."

        oDlg = SutekhDialog(sName, self.parent,
                gtk.DIALOG_DESTROY_WITH_PARENT)
        oDlg.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        oDlg.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)

        oDlg.connect("response", self.handle_response)

        # pylint: disable-msg=W0201
        # no point in defining this in __init__
        self._aButtons = []
        aColsInUse = self.get_cols_in_use()
        # pylint: disable-msg=E1101
        # pylint doesn't detect vbox's methods
        for sName in self._dCols:
            oBut = gtk.CheckButton(sName)
            oBut.set_active(sName in aColsInUse)
            oDlg.vbox.pack_start(oBut)
            self._aButtons.append(oBut)

        oDlg.vbox.pack_start(gtk.HSeparator())

        oIter = self._dModes.iterkeys()
        for sName in oIter:
            self._oFirstBut = gtk.RadioButton(None, sName, False)
            if self._iShowMode == self._dModes[sName]:
                self._oFirstBut.set_active(True)
            oDlg.vbox.pack_start(self._oFirstBut, expand=False)
            break
        for sName in oIter:
            oBut = gtk.RadioButton(self._oFirstBut, sName)
            oDlg.vbox.pack_start(oBut, expand=False)
            if self._iShowMode == self._dModes[sName]:
                oBut.set_active(True)

        oDlg.show_all()

        return oDlg

    # Actions

    def handle_response(self, oDlg, oResponse):
        """Handle user response from the dialog"""
        if oResponse == gtk.RESPONSE_CANCEL:
            oDlg.destroy()
        elif oResponse == gtk.RESPONSE_OK:
            aColsInUse = []
            for oBut in self._aButtons:
                if oBut.get_active():
                    sLabel = oBut.get_label()
                    aColsInUse.append(sLabel)
            for oBut in self._oFirstBut.get_group():
                sName = oBut.get_label()
                if oBut.get_active():
                    self._iShowMode = self._dModes[sName]

            self.set_cols_in_use(aColsInUse)
            oDlg.destroy()

    def set_cols_in_use(self, aCols):
        """Add columns to the view"""
        # pylint: disable-msg=W0612
        # iDir is returned, although we don't need it
        iSortCol, iDir = self.model.get_sort_column_id()
        # pylint: enable-msg=W0612
        if iSortCol is not None and iSortCol > 1:
            # We're changing the columns, so restore sorting to default
            self.model.set_sort_column_id(0, 0)

        for oCol in self._get_col_objects():
            self.view.remove_column(oCol)

        for iNum, sCol in enumerate(aCols):
            oCell = CellRendererIcons()
            oColumn = gtk.TreeViewColumn(sCol, oCell)
            oColumn.set_cell_data_func(oCell, self._dCols[sCol])
            oColumn.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
            oColumn.set_fixed_width(self._dWidths.get(sCol, 100))
            oColumn.set_resizable(True)
            self.view.insert_column(oColumn, iNum + 3)
            oColumn.set_sort_column_id(iNum + 3)
            self.model.set_sort_func(iNum + 3, self.sort_column,
                    self._dSortDataFuncs[sCol])

    def get_cols_in_use(self):
        """Get which extra columns have been added to view"""
        return [oCol.get_property("title") for oCol in self._get_col_objects()]

    def _get_col_objects(self):
        """Get the actual TreeColumn in the view"""
        return [oCol for oCol in self.view.get_columns() if
                oCol.get_property("title") in self._dCols]

# pylint: disable-msg=C0103
# accept plugin name
plugin = ExtraCardViewColumns



# ImportFromZipFile.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details
"""Plugin to import selected card sets from a zip file"""

import gtk
import os
from logging import Logger
from sutekh.core.SutekhObjects import PhysicalCardSet
from sutekh.gui.PluginManager import CardListPlugin
from sutekh.gui.SutekhDialog import SutekhDialog, do_complaint_error
from sutekh.gui.SutekhFileWidget import SutekhFileDialog
from sutekh.gui.ProgressDialog import ProgressDialog, SutekhCountLogHandler
from sutekh.gui.ScrolledList import ScrolledList
from sutekh.io.ZipFileWrapper import ZipFileWrapper

def _do_rename_parent(sOldName, sNewName, dRemaining):
    """Handle the renaming of a parent card set in the unprocessed list."""
    dResult = {}
    for sName, tInfo in dRemaining.iteritems():
        if tInfo[2] == sOldName:
            dResult[sName] = (tInfo[0], tInfo[1], sNewName)
        else:
            dResult[sName] = tInfo
    return dResult

class ImportFromZipFile(CardListPlugin):
    """Extract selected card sets from a zip file."""

    dTableVersions = {}
    aModelsSupported = ["MainWindow"]

    # Dialog and Menu Item Creation

    def get_menu_item(self):
        """Register on the Plugins menu"""
        if not self.check_versions() or not self.check_model_type():
            return None

        oImport = gtk.MenuItem("Import Card Set(s) from zip file")
        oImport.connect("activate", self.make_dialog)
        return ('Import Card Set', oImport)

    # Menu responses

    # pylint: disable-msg=W0613
    # oWidget needed by gtk function signature

    def make_dialog(self, oWidget):
        """Create the dialog used to select the zip file"""
        sName = "Select zip file to import from."

        oDlg = SutekhFileDialog(self.parent, sName,
                oAction=gtk.FILE_CHOOSER_ACTION_OPEN,
                oButtons=(gtk.STOCK_OK, gtk.RESPONSE_OK,
                    gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
        oDlg.set_name("Sutekh.dialog")
        oDlg.add_filter_with_pattern('Zip Files', ['*.zip', '*.ZIP'])

        oDlg.connect("response", self.handle_response)
        oDlg.set_local_only(True)
        oDlg.set_select_multiple(False)
        oDlg.show_all()

        return oDlg

    # pylint: enable-msg=W0613

    def handle_response(self, oDlg, oResponse):
        """Handle response from the import dialog"""
        if oResponse == gtk.RESPONSE_OK:
            sFile = oDlg.get_filename()
            oDlg.destroy()

            if not os.path.exists(sFile):
                do_complaint_error("Backup file %s does not seem to exist."
                        % sFile)
                return
            oFile = ZipFileWrapper(sFile)
            dList = oFile.get_all_entries()
            dSelected = {}
            # Ask user to select entries to import
            oSelDlg = SutekhDialog("Select Card Sets to Import", self.parent,
                    gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                    (gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL,
                        gtk.RESPONSE_CANCEL))

            oScrolledList = ScrolledList('Available Card Sets')
            # pylint: disable-msg=E1101
            # vbox confuses pylint
            oSelDlg.vbox.pack_start(oScrolledList)
            oScrolledList.set_size_request(150, 300)
            oScrolledList.fill_list(sorted(dList))
            oResponse = oSelDlg.run()
            # Extract selected cards from the dialog
            for sName in oScrolledList.get_selection():
                dSelected[sName] = dList[sName]
            oSelDlg.destroy()
            if oResponse == gtk.RESPONSE_OK and len(dSelected) > 0:
                self.do_read_list(oFile, dSelected)
        else:
            oDlg.destroy()

    def do_read_list(self, oFile, dSelected):
        """Read the selected list of card sets"""
        oLogHandler = SutekhCountLogHandler()
        oProgressDialog = ProgressDialog()
        oProgressDialog.set_description("Importing Files")
        oLogger = Logger('Read zip file')
        oLogger.addHandler(oLogHandler)
        oLogHandler.set_dialog(oProgressDialog)
        oLogHandler.set_total(len(dSelected))
        oProgressDialog.show()
        bDone = False
        while not bDone:
            dSelected = self._read_heart(oFile, dSelected, oLogger)
            bDone = len(dSelected) == 0
        oProgressDialog.destroy()

    def _read_heart(self, oFile, dSelected, oLogger):
        """Heart of the reading loop - ensure we read parents before
           children, and correct for renames that occur."""
        # pylint: disable-msg=W0703
        # we really do want all the exceptions
        dRemaining = {}
        dRenames = {}
        for sName, tInfo in dSelected.iteritems():
            sFilename, bParentExists, sParentName = tInfo
            # We aim to ensure that we always keep loaded card sets together,
            # so if the parent name clashes, we want to follow the renamed
            # parent, rather than be added to the existing card set of the
            # same name
            if sParentName is not None and sParentName in dSelected:
                # Do have a parent to look at, so skip for now
                dRemaining[sName] = tInfo
                continue
            elif sParentName is not None and sParentName in dRenames:
                # Renamed parent, so adjust this - this will also correctly
                # handle cards that no longer need a parent, since it will
                # set sParentName to None
                sParentName = dRenames[sParentName]
            elif not bParentExists:
                # Parent doesn't exist, but isn't in the list, so turn into
                # top level card set
                sParentName = None
            try:
                oCardSetHolder = oFile.read_single_card_set(sFilename)
                oLogger.info('Read %s' % sName)
                # Check for whether we're overwriting something
                # or not
                if PhysicalCardSet.selectBy(name=sName).count() != 0:
                    # Ask the user whether to rename, replace or cancel
                    sNewName = self._query_rename(sName)
                    dRenames[sName] = sNewName
                    # Check for cardsets that were waiting for us
                    dRemaining = _do_rename_parent(sName, sNewName, dRemaining)
                    if sNewName:
                        # rename
                        oCardSetHolder.name = sNewName
                    else:
                        # We skip this card set
                        continue
                # Ensure we use the right name for the parent
                oCardSetHolder.parent = sParentName
                oCardSetHolder.create_pcs(self.cardlookup)
                self.reload_pcs_list()
            except Exception, oException:
                sMsg = "Failed to import card set %s.\n\n%s" % (sName,
                        oException)
                do_complaint_error(sMsg)
        return dRemaining

    def _query_rename(self, sOldName):
        """Request a new name for the card set."""
        sNewName = None
        bRenamed = False
        oDlg = SutekhDialog("Card Set Name Exists", self.parent,
                gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                (gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL,
                    gtk.RESPONSE_CANCEL))
        # pylint: disable-msg=E1101
        # vbox confuses pylint
        oLabel = gtk.Label()
        oLabel.set_markup('Card Set %s already.\nPlease choose a new name or\n'
                'press Cancel to skip this card set' % sOldName)
        oDlg.vbox.pack_start(oLabel)
        oEntry = gtk.Entry(50)
        oDlg.vbox.pack_start(oEntry)
        # Need this so entry box works as expected
        oEntry.connect("activate", oDlg.response, gtk.RESPONSE_OK)
        oDlg.vbox.show_all()
        while not bRenamed:
            iResponse = oDlg.run()
            if iResponse == gtk.RESPONSE_OK:
                sNewName = oEntry.get_text().strip()
                if not sNewName:
                    do_complaint_error('No new name given')
                    # go around again
                    continue
                elif PhysicalCardSet.selectBy(name=sNewName).count() != 0:
                    do_complaint_error('New name %s is already in use'
                            % sNewName)
                    # go around again
                    continue
            else:
                sNewName = None
            # we got here, so we're done
            bRenamed = True
        oDlg.destroy()
        return sNewName


# pylint: disable-msg=C0103
# accept plugin name
plugin = ImportFromZipFile



# test_WhiteWolfParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2007 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Test the white wolf card reader"""

from sutekh.tests.TestCore import SutekhTest
from sutekh.core.SutekhObjects import AbstractCard, IAbstractCard, \
        IPhysicalCard, IClan, IDisciplinePair, ICardType, ISect, ITitle, \
        ICreed, IVirtue, IExpansion, IRarity, IRarityPair
from sqlobject import SQLObjectNotFound
import unittest

class WhiteWolfParserTests(SutekhTest):
    """Test class for the white wolf card reader.

       Check the parsing done in SutekhTest setup and verify the results
       are correct.
       """
    aExpectedCards = [
        u".44 Magnum", u"AK-47", u"Aabbt Kindred",
        u"Aaron Bathurst", u"Aaron Duggan, Cameron's Toady",
        u"Aaron's Feeding Razor", u"Abandoning the Flesh",
        u"Abbot", u"Abd al-Rashid", u"Abdelsobek",
        u"Abebe", u"Abjure", u"Ablative Skin",
        u"Abombwe", u"Aeron", u"Aire of Elation", u"Akram",
        u"Alan Sovereign", u"Alan Sovereign (Advanced)", u"Alexandra",
        u"Alfred Benezri", u"Ambrogino Giovanni", u'Amisa',
        u"Anastasz di Zagreb", u"Angelica, The Canonicus",
        u'Anna "Dictatrix11" Suljic', u"Anson", u"Bronwen", u"Cedric",
        u"Cesewayo", u'Earl "Shaka74" Deams', u"Gracis Nostinus",
        u'Inez "Nurse216" Villagrande', u"Kabede Maru",
        u"Kemintiri (Advanced)", u"L\xe1z\xe1r Dobrescu",
        u"Predator's Communion", u"Sha-Ennu", u"The Path of Blood",
        u"Yvette, The Hopeless",
        ]

    def test_basic(self):
        """Basic WW list parser tests"""
        # pylint: disable-msg=E1101, R0915, R0914
        # E1101: SQLObject + PyProtocols magic confuses pylint
        # R0915, R0914: Want a long, sequential test case to minimise
        aCards = sorted(list(AbstractCard.select()), cmp=lambda oC1, oC2:
                cmp(oC1.name, oC2.name))

        # Check card names
        self.assertEqual([oC.name for oC in aCards], self.aExpectedCards)

        # Check Magnum
        # pylint: disable-msg=C0103
        # o44 is OK here
        o44 = IAbstractCard(".44 Magnum")
        self.assertEqual(o44.canonicalName, u".44 magnum")
        self.assertEqual(o44.name, u".44 Magnum")
        self.failUnless(o44.text.startswith(u"Weapon, gun.\n2R"))
        self.failUnless(o44.text.endswith(u"each combat."))
        self.assertEqual(o44.group, None)
        self.assertEqual(o44.capacity, None)
        self.assertEqual(o44.cost, 2)
        self.assertEqual(o44.life, None)
        self.assertEqual(o44.costtype, 'pool')
        self.assertEqual(o44.level, None)

        # pylint: enable-msg=C0103
        oCommon = IRarity('Common')
        oJyhad = IExpansion('Jyhad')
        oVTES = IExpansion('VTES')

        self.assertTrue(oCommon in [oP.rarity for oP in o44.rarity])
        self.assertTrue(oJyhad in [oP.expansion for oP in o44.rarity])
        self.assertTrue(oVTES in [oP.expansion for oP in o44.rarity])

        self.assertTrue(IRarityPair(('VTES', 'Common')) in o44.rarity)


        # Find some discipline pairs
        oFortInf = IDisciplinePair((u"Fortitude", u"inferior"))
        oFortSup = IDisciplinePair((u"Fortitude", u"superior"))
        oQuiSup = IDisciplinePair((u"Quietus", u"superior"))
        oCelInf = IDisciplinePair((u"Celerity", u"inferior"))
        oAusInf = IDisciplinePair((u"Auspex", u"inferior"))
        oAusSup = IDisciplinePair((u"Auspex", u"superior"))
        oPreSup = IDisciplinePair((u"Presence", u"superior"))
        oObfSup = IDisciplinePair((u"Obfuscate", u"superior"))
        oAboInf = IDisciplinePair((u"Abombwe", u"inferior"))
        oAboSup = IDisciplinePair((u"Abombwe", u"superior"))

        # Check Dobrescu
        oDob = IAbstractCard(u"L\xe1z\xe1r Dobrescu")
        self.assertEqual(oDob.canonicalName, u"l\xe1z\xe1r dobrescu")
        self.assertEqual(oDob.name, u"L\xe1z\xe1r Dobrescu")
        self.failUnless(oDob.text.startswith(
            u"Independent: L\xe1z\xe1r may move"))
        self.failUnless(oDob.text.endswith(u"as a (D) action."))
        self.assertEqual(oDob.group, 2)
        self.assertEqual(oDob.capacity, 3)
        self.assertEqual(oDob.cost, None)
        self.assertEqual(oDob.life, None)
        self.assertEqual(oDob.costtype, None)
        self.assertEqual(oDob.level, None)

        self.failUnless(IClan('Ravnos') in oDob.clan)
        self.assertEqual(len(oDob.discipline), 1)
        self.failUnless(oFortInf in oDob.discipline)
        self.assertEqual(len(oDob.cardtype), 1)
        self.failUnless(ICardType('Vampire') in oDob.cardtype)
        self.failUnless(ISect('Independent') in oDob.sect)

        self.assertEqual(len(oDob.rulings), 1)
        oRuling = oDob.rulings[0]
        self.assertTrue(oRuling.text.startswith("Cannot use his special"))
        self.assertTrue(oRuling.text.endswith("uncontrolled region."))
        self.assertEqual(oRuling.code, "[LSJ19990215]")

        # Check Abstract and Physical expansions match
        for oAbs in AbstractCard.select():
            aExps = [oPair.expansion for oPair in oAbs.rarity]
            for oExp in aExps:
                try:
                    oPair = IPhysicalCard((oAbs, oExp))
                except SQLObjectNotFound:
                    self.fail(
                        "Missing physical card %s from expansion %s"
                        % (oAbs.name, oExp.name)
                    )

        # Check Yvette
        oYvette = IAbstractCard(u"Yvette, The Hopeless")
        self.assertEqual(oYvette.canonicalName, u"yvette, the hopeless")
        self.assertEqual(oYvette.name, u"Yvette, The Hopeless")
        self.failUnless(oYvette.text.startswith("Camarilla."))
        self.assertEqual(oYvette.group, 3)
        self.assertEqual(oYvette.capacity, 3)
        self.assertEqual(oYvette.cost, None)
        self.assertEqual(oYvette.life, None)
        self.assertEqual(oYvette.costtype, None)
        self.assertEqual(oYvette.level, None)

        self.failUnless(IClan('Toreador') in oYvette.clan)
        self.assertEqual(len(oYvette.clan), 1)
        self.assertEqual(len(oYvette.discipline), 2)
        self.failUnless(oFortInf not in oYvette.discipline)
        self.failUnless(oCelInf in oYvette.discipline)
        self.failUnless(oAusInf in oYvette.discipline)
        self.assertEqual(len(oYvette.cardtype), 1)
        self.failUnless(ICardType('Vampire') in oYvette.cardtype)
        self.failUnless(ISect('Camarilla') in oYvette.sect)

        # Check Sha-Ennu

        oShaEnnu = IAbstractCard(u"Sha-Ennu")
        self.assertEqual(oShaEnnu.canonicalName, u"sha-ennu")
        self.assertEqual(oShaEnnu.name, u"Sha-Ennu")
        self.failUnless(oShaEnnu.text.startswith("Sabbat regent:"))
        self.failUnless(oShaEnnu.text.endswith("+2 bleed."))
        self.assertEqual(oShaEnnu.group, 4)
        self.assertEqual(oShaEnnu.capacity, 11)
        self.assertEqual(oShaEnnu.cost, None)
        self.assertEqual(oShaEnnu.life, None)
        self.assertEqual(oShaEnnu.costtype, None)
        self.assertEqual(oShaEnnu.level, None)

        self.failUnless(IClan('Tzimisce') in oShaEnnu.clan)
        self.assertEqual(len(oShaEnnu.clan), 1)
        self.assertEqual(len(oShaEnnu.discipline), 6)
        self.failUnless(oAusSup in oShaEnnu.discipline)
        self.failUnless(oAusInf not in oShaEnnu.discipline)
        self.assertEqual(len(oShaEnnu.cardtype), 1)
        self.failUnless(ICardType('Vampire') in oShaEnnu.cardtype)
        self.failUnless(ISect('Sabbat') in oShaEnnu.sect)
        self.failUnless(ITitle('Regent') in oShaEnnu.title)
        self.assertTrue(IRarityPair(('Third', 'Vampire')) in oShaEnnu.rarity)
        self.assertFalse(IRarityPair(('VTES', 'Common')) in oShaEnnu.rarity)
        self.assertFalse(IRarityPair(('VTES', 'Vampire')) in oShaEnnu.rarity)

        # Check Kabede

        oKabede = IAbstractCard(u"Kabede Maru")
        self.assertEqual(oKabede.canonicalName, u"kabede maru")
        self.assertEqual(oKabede.name, u"Kabede Maru")
        self.failUnless(oKabede.text.startswith("Laibon magaji:"))
        self.failUnless(oKabede.text.endswith("affect Kabede.)"))
        self.assertEqual(oKabede.group, 5)
        self.assertEqual(oKabede.capacity, 9)
        self.assertEqual(oKabede.cost, None)
        self.assertEqual(oKabede.life, None)
        self.assertEqual(oKabede.costtype, None)
        self.assertEqual(oKabede.level, None)

        self.failUnless(IClan('Assamite') in oKabede.clan)
        self.assertEqual(len(oKabede.clan), 1)
        self.assertEqual(len(oKabede.discipline), 6)
        self.failUnless(oAusSup in oKabede.discipline)
        self.failUnless(oQuiSup in oKabede.discipline)
        self.failUnless(oAboInf in oKabede.discipline)
        self.failUnless(oAboSup not in oKabede.discipline)
        self.assertEqual(len(oKabede.cardtype), 1)
        self.failUnless(ICardType('Vampire') in oKabede.cardtype)
        self.failUnless(ISect('Laibon') in oKabede.sect)
        self.failUnless(ITitle('Magaji') in oKabede.title)
        self.assertTrue(IRarityPair(('LotN', 'Uncommon')) in oKabede.rarity)

        # Check Predator's Communion
        oPredComm = IAbstractCard(u"Predator's Communion")
        self.assertEqual(oPredComm.canonicalName, u"predator's communion")
        self.assertEqual(oPredComm.name, u"Predator's Communion")
        self.failUnless(oPredComm.text.startswith("[abo] [REFLEX]"))
        self.failUnless(oPredComm.text.endswith("untaps."))
        self.assertEqual(oPredComm.group, None)
        self.assertEqual(oPredComm.capacity, None)
        self.assertEqual(oPredComm.cost, None)
        self.assertEqual(oPredComm.life, None)
        self.assertEqual(oPredComm.costtype, None)
        self.assertEqual(oPredComm.level, None)

        self.assertEqual(len(oPredComm.discipline), 1)
        self.failUnless(oAboSup in oPredComm.discipline)
        self.assertEqual(len(oPredComm.cardtype), 2)
        self.failUnless(ICardType('Reaction') in oPredComm.cardtype)
        self.failUnless(ICardType('Reflex') in oPredComm.cardtype)
        self.assertTrue(IRarityPair(('LoB', 'Common')) in oPredComm.rarity)

        # Check Earl
        oEarl = IAbstractCard(u'Earl "Shaka74" Deams')
        self.assertEqual(oEarl.canonicalName, u'earl "shaka74" deams')
        self.assertEqual(oEarl.name, u'Earl "Shaka74" Deams')
        self.failUnless(oEarl.text.startswith("Earl gets +1 stealth"))
        self.failUnless(oEarl.text.endswith("[1 CONVICTION]."))
        self.assertEqual(oEarl.group, 4)
        self.assertEqual(oEarl.capacity, None)
        self.assertEqual(oEarl.cost, None)
        self.assertEqual(oEarl.life, 6)
        self.assertEqual(oEarl.costtype, None)
        self.assertEqual(oEarl.level, None)

        self.failUnless(ICreed('Visionary') in oEarl.creed)
        self.assertEqual(len(oEarl.creed), 1)
        self.assertEqual(len(oEarl.virtue), 3)
        self.failUnless(IVirtue('Martyrdom') in oEarl.virtue)
        self.failUnless(IVirtue('Judgment') in oEarl.virtue)
        self.failUnless(IVirtue('Vision') in oEarl.virtue)
        self.assertEqual(len(oEarl.cardtype), 1)
        self.failUnless(ICardType('Imbued') in oEarl.cardtype)
        self.assertTrue(IRarityPair(('NoR', 'Uncommon')) in oEarl.rarity)

        # Check Aire
        oAire = IAbstractCard("Aire of Elation")
        self.assertEqual(oAire.canonicalName, u"aire of elation")
        self.assertEqual(oAire.name, u"Aire of Elation")
        self.failUnless(oAire.text.startswith("You cannot play another"))
        self.failUnless(oAire.text.endswith("is Toreador."))
        self.assertEqual(oAire.group, None)
        self.assertEqual(oAire.capacity, None)
        self.assertEqual(oAire.cost, 1)
        self.assertEqual(oAire.life, None)
        self.assertEqual(oAire.costtype, 'blood')
        self.assertEqual(oAire.level, None)

        self.assertEqual(len(oAire.discipline), 1)
        self.failUnless(oPreSup in oAire.discipline)
        self.assertEqual(len(oAire.cardtype), 1)
        self.failUnless(ICardType("Action Modifier") in oAire.cardtype)

        self.assertTrue(IRarityPair(('DS', 'Common')) in oAire.rarity)
        self.assertTrue(IRarityPair(('FN', 'Precon')) in oAire.rarity)
        self.assertTrue(IRarityPair(('CE', 'Common')) in oAire.rarity)
        self.assertTrue(IRarityPair(('CE', 'Precon')) in oAire.rarity)
        self.assertTrue(IRarityPair(('Anarchs', 'Precon')) in oAire.rarity)
        self.assertTrue(IRarityPair(('KMW', 'Precon')) in oAire.rarity)

        # Abjure
        oAbjure = IAbstractCard("Abjure")
        self.assertEqual(oAbjure.canonicalName, u"abjure")
        self.assertEqual(oAbjure.name, u"Abjure")
        self.failUnless(oAbjure.text.startswith("[COMBAT] Tap this"))
        self.failUnless(oAbjure.text.endswith("or ash heap."))
        self.assertEqual(oAbjure.group, None)
        self.assertEqual(oAbjure.capacity, None)
        self.assertEqual(oAbjure.cost, None)
        self.assertEqual(oAbjure.life, None)
        self.assertEqual(oAbjure.costtype, None)
        self.assertEqual(oAbjure.level, None)

        self.assertEqual(len(oAbjure.virtue), 1)
        self.failUnless(IVirtue('Redemption') in oAbjure.virtue)
        self.assertEqual(len(oAbjure.cardtype), 1)
        self.failUnless(ICardType('Power') in oAbjure.cardtype)

        # Abombwe Master Card
        oAbo = IAbstractCard('Abombwe')

        self.assertEqual(oAbo.canonicalName, 'abombwe')
        self.assertEqual(oAbo.name, 'Abombwe')
        self.failUnless(oAbo.text.startswith("Master: Discipline. Trifle"))
        self.failUnless(oAbo.text.endswith("superior Abombwe."))
        self.assertEqual(oAbo.group, None)
        self.assertEqual(oAbo.capacity, 1)
        self.assertEqual(oAbo.cost, None)
        self.assertEqual(oAbo.life, None)
        self.assertEqual(oAbo.costtype, None)
        self.assertEqual(oAbo.level, None)

        self.assertEqual(len(oAbo.discipline), 0)
        self.assertEqual(len(oAbo.cardtype), 1)
        self.failUnless(ICardType('Master') in oAbo.cardtype)
        self.assertEqual(len(oAbo.rulings), 0)

        # Ablative Skin Card
        oAblat = IAbstractCard('Ablative Skin')

        self.assertEqual(oAblat.canonicalName, 'ablative skin')
        self.assertEqual(oAblat.name, 'Ablative Skin')
        self.failUnless(oAblat.text.startswith("+1 stealth action"))
        self.failUnless(oAblat.text.endswith("damage in combat in this way."))
        self.assertEqual(oAblat.group, None)
        self.assertEqual(oAblat.cost, None)
        self.assertEqual(oAblat.life, None)
        self.assertEqual(oAblat.costtype, None)
        self.assertEqual(oAblat.level, None)

        self.assertEqual(len(oAblat.discipline), 1)
        self.failUnless(oFortSup in oAblat.discipline)
        self.assertEqual(len(oAblat.cardtype), 1)
        self.failUnless(ICardType('Action') in oAblat.cardtype)
        self.assertEqual(len(oAblat.rulings), 1)
        oRuling = oAblat.rulings[0]
        self.assertTrue(oRuling.text.startswith("Cannot be used to prevent"))
        self.assertTrue(oRuling.text.endswith("Blood Fury)."))
        self.assertEqual(oRuling.code, "[LSJ19990216]")

        # Check Independent titles
        oAmbrig = IAbstractCard(u"Ambrogino Giovanni")
        self.failUnless(ITitle('Independent with 1 vote') in oAmbrig.title)
        oAmisa = IAbstractCard(u"Amisa")
        self.failUnless(ITitle('Independent with 2 votes') in oAmisa.title)

        # Check Kemintiri
        oKemintiri = IAbstractCard(u"Kemintiri (Advanced)")
        self.assertEqual(oKemintiri.canonicalName, u"kemintiri (advanced)")
        self.assertEqual(oKemintiri.name, u"Kemintiri (Advanced)")
        self.failUnless(oKemintiri.text.startswith("Advanced, Independent."))
        self.assertEqual(oKemintiri.group, 2)
        self.assertEqual(oKemintiri.capacity, 10)
        self.assertEqual(oKemintiri.cost, None)
        self.assertEqual(oKemintiri.life, None)
        self.assertEqual(oKemintiri.costtype, None)
        self.assertEqual(oKemintiri.level, 'advanced')

        self.failUnless(IClan('Follower of Set') in oKemintiri.clan)
        self.assertEqual(len(oKemintiri.clan), 1)
        self.assertEqual(len(oKemintiri.discipline), 6)
        self.failUnless(oAusInf in oKemintiri.discipline)
        self.failUnless(oObfSup in oKemintiri.discipline)
        self.failUnless(oPreSup in oKemintiri.discipline)
        self.assertEqual(len(oKemintiri.cardtype), 1)
        self.failUnless(ICardType('Vampire') in oKemintiri.cardtype)
        self.failUnless(ISect('Independent') in oKemintiri.sect)
        # Make sure we're not picking up the Merged title
        self.assertEqual(len(oKemintiri.title), 0)
        self.assertTrue(IRarityPair(('KMW', 'Uncommon')) in oKemintiri.rarity)


if __name__ == "__main__":
    unittest.main()



# TestData.py
# -*- coding: cp1252 -*-
# vim:fileencoding=cp1252 ai ts=4 sts=4 et sw=4
# Based on the card data published by WhiteWolf at:
#  * http://www.white-wolf.com/vtes/index.php?line=cardlist
#  * http://www.white-wolf.com/vtes/index.php?line=rulings

"""Data used for test cases"""
# pylint: disable-msg=C0103
# don't care about naming conventions here

TEST_CARD_LIST = """
<html>
<head><title>Sutekh Test Cards</title></head>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
<body>
<table>
<tr><td>

<p>
<a name="card_1"><span class="cardname">.44 Magnum</span></a>
<span class="exp">[Jyhad:C, VTES:C, Sabbat:C, SW:PB, CE:PTo3, LoB:PO3]</span><br />
<table><tr><td><span class="key">Cardtype:</span> </td><td>Equipment<br /></td></tr>
<tr><td><span class="key">Cost:</span> </td><td>2 pool<br /></td></tr>
<tr><td colspan=2><b>Weapon, gun.</b><br />
<n>2R damage each strike, with an optional maneuver each combat.<br />
<tr><td><span class="key">Artist:</span> </td><td>Né Né Thomas; Greg Simanson<br /></td></tr>
</table>
</p>
<p>
<a name="card_2"><span class="cardname">AK-47</span></a>
<span class="exp">[LotN:R]</span><br />
<table><tr><td><span class="key">Cardtype:</span> </td><td>Equipment<br /></td></tr>
<tr><td><span class="key">Cost:</span> </td><td>5 pool<br /></td></tr>
<tr><td colspan=2><b>Weapon. Gun.</b><br />
<n>2R damage each strike, with an optional maneuver <span class="errata">{each combat}</span>. When bearer strikes with this gun, he or she gets an optional additional strike this round, only usable to strike with this gun.<br />
<tr><td><span class="key">Artist:</span> </td><td>Franz Vohwinkel<br /></td></tr>
</table>
</p>
<p>
<a name="card_3"><span class="cardname">Aabbt Kindred</span></a>
<span class="exp">[FN:U2]</span><br />
<table><tr><td><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td><span class="key">Clan:</span> </td><td>Follower of Set<br /></td></tr>
<tr><td><span class="key">Group:</span> </td><td>2<br /></td></tr>
<tr><td><span class="key">Capacity:</span> </td><td>4<br /></td></tr>
<tr><td><span class="key">Discipline:</span> </td><td>for pre ser<br /></td></tr>
<tr><td colspan=2><b>Independent:</b> <n>Aabbt Kindred cannot perform (D) actions unless Nefertiti is ready. Aabbt Kindred can prevent 1 damage each combat. Aabbt Kindred are not unique and do not contest.<br />
<tr><td><span class="key">Artist:</span> </td><td>Lawrence Snelly<br /></td></tr>
</table>
</p>
<p>
<a name="card_4"><span class="cardname">Aaron Bathurst</span></a>
<span class="exp">[Third:V]</span><br />
<table><tr><td><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td><span class="key">Clan:</span> </td><td>Nosferatu antitribu<br /></td></tr>
<tr><td><span class="key">Group:</span> </td><td>4<br /></td></tr>
<tr><td><span class="key">Capacity:</span> </td><td>4<br /></td></tr>
<tr><td><span class="key">Discipline:</span> </td><td>for obf pot<br /></td></tr>
<tr><td colspan=2><b>Sabbat.</b><br />
<tr><td><span class="key">Artist:</span> </td><td>Rik Martin<br /></td></tr>
</table>
</p>
<p>
<a name="card_5"><span class="cardname">Aaron Duggan, Cameron's Toady</span></a>
<span class="exp">[Sabbat:V, SW:U]</span><br />
<table><tr><td><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td><span class="key">Clan:</span> </td><td>Lasombra<br /></td></tr>
<tr><td><span class="key">Group:</span> </td><td>2<br /></td></tr>
<tr><td><span class="key">Capacity:</span> </td><td>2<br /></td></tr>
<tr><td><span class="key">Discipline:</span> </td><td>obt<br /></td></tr>
<tr><td colspan=2><b>Sabbat</b><br />
<tr><td><span class="key">Artist:</span> </td><td>Eric LaCombe<br /></td></tr>
</table>
</p>
<p>
<a name="card_6"><span class="cardname">Aaron's Feeding Razor</span></a>
<span class="exp">[Jyhad:R, VTES:R, CE:R]</span><br />
<table><tr><td><span class="key">Cardtype:</span> </td><td>Equipment<br /></td></tr>
<tr><td><span class="key">Cost:</span> </td><td>1 pool<br /></td></tr>
<tr><td colspan=2><b>Unique equipment.</b><br />
<n>If the vampire with this equipment successfully hunts, he or she gains 1 additional blood.<br />
<tr><td><span class="key">Artist:</span> </td><td>Thomas Nairb; Christopher Rush<br /></td></tr>
</table>
</p>
<p>
<a name="card_7"><span class="cardname">Abandoning the Flesh</span></a>
<span class="exp">[CE:R, Third:R]</span><br />
<table><tr><td><span class="key">Cardtype:</span> </td><td>Reaction/Combat<br /></td></tr>
<tr><td><span class="key">Discipline:</span> </td><td>Dementation<br /></td></tr>
<tr><td colspan=2><b>Only usable by a vampire being burned. Usable by a vampire in torpor.</b><br />
<n>[dem] Remove this vampire from the game instead (diablerie, if any, is still successful), and put this card into play. You may not play this card if you already have an Abandoning the Flesh in play. You may tap this card when a vampire with Dementation is bleeding to give that vampire +1 bleed for the current action.<br />
<tr><td><span class="key">Artist:</span> </td><td>Steve Ellis<br /></td></tr>
</table>
</p>
<p>
<a name="card_8"><span class="cardname">Abbot</span></a>
<span class="exp">[Third:U]</span><br />
<table><tr><td><span class="key">Cardtype:</span> </td><td>Action<br /></td></tr>
<tr><td colspan=2><b>+1 stealth action. Requires a Sabbat vampire.</b><br />
<n>Put this card on this acting Sabbat vampire and untap him or her. This Sabbat vampire gets +1 intercept against (D) actions directed at his or her controller. A vampire may have only one Abbot.<br />
<tr><td><span class="key">Artist:</span> </td><td>John Bridges<br /></td></tr>
</table>
</p>
<p>
<a name="card_9"><span class="cardname">Abd al-Rashid</span></a>
<span class="exp">[AH:V3, FN:PA]</span><br />
<table><tr><td><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td><span class="key">Clan:</span> </td><td>Assamite<br /></td></tr>
<tr><td><span class="key">Group:</span> </td><td>2<br /></td></tr>
<tr><td><span class="key">Capacity:</span> </td><td>5<br /></td></tr>
<tr><td><span class="key">Discipline:</span> </td><td>obf CEL QUI<br /></td></tr>
<tr><td colspan=2><b>Independent:</b> <n>(Blood Cursed)<br />
<tr><td><span class="key">Artist:</span> </td><td>Tom Wänerstrand<br /></td></tr>
</table>
</p>
<p>
<a name="card_10"><span class="cardname">Abdelsobek</span></a>
<span class="exp">[LotN:U]</span><br />
<table><tr><td><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td><span class="key">Clan:</span> </td><td>Follower of Set<br /></td></tr>
<tr><td><span class="key">Group:</span> </td><td>5<br /></td></tr>
<tr><td><span class="key">Capacity:</span> </td><td>5<br /></td></tr>
<tr><td><span class="key">Discipline:</span> </td><td>for nec obf pre ser<br /></td></tr>
<tr><td colspan=2><b>Independent:</b> <n>Abdelsobek can untap a vampire or mummy you control as a +1 stealth action.<br />
<tr><td><span class="key">Artist:</span> </td><td>Ken Meyer, Jr.<br /></td></tr>
</table>
</p>
<p>
<a name="card_11"><span class="cardname">Abebe</span></a>
<span class="exp">[LoB:U]</span><br />
<table><tr><td><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td><span class="key">Clan:</span> </td><td>Samedi<br /></td></tr>
<tr><td><span class="key">Group:</span> </td><td>4<br /></td></tr>
<tr><td><span class="key">Capacity:</span> </td><td>4<br /></td></tr>
<tr><td><span class="key">Discipline:</span> </td><td>nec obf thn<br /></td></tr>
<tr><td colspan=2><b>Independent.</b><br />
<tr><td><span class="key">Artist:</span> </td><td>James Stowe<br /></td></tr>
</table>
</p>
<p>
<a name="card_12"><span class="cardname">Abjure</span></a>
<span class="exp">[NoR:R]</span><br />
<table><tr><td><span class="key">Cardtype:</span> </td><td>Power<br /></td></tr>
<tr><td><span class="key">Virtue:</span> </td><td>Redemption<br /></td></tr>
<tr><td colspan=2><n>[COMBAT] Tap this imbued before range is determined to end a combat between a monster and a mortal. If the mortal is a minion other than this imbued, you may move a conviction to this imbued from your hand or ash heap.<br />
<tr><td><span class="key">Artist:</span> </td><td>Brian LeBlanc<br /></td></tr>
</table>
</p>
<p>
<a name="card_13"><span class="cardname">Ablative Skin</span></a>
<span class="exp">[Sabbat:R, SW:R, Third:R]</span><br />
<table><tr><td><span class="key">Cardtype:</span> </td><td>Action<br /></td></tr>
<tr><td><span class="key">Discipline:</span> </td><td>Fortitude<br /></td></tr>
<tr><td colspan=2><b>+1 stealth action.</b><br />
<n>[for] Put this card on the acting vampire and put 3 ablative counters on this card. While in combat, this vampire may remove any number of ablative counters from this card to prevent that amount of non-aggravated damage. Burn this card when it has no more ablative counters.<br />
<b>[FOR] As above, but this vampire may also prevent aggravated damage in combat in this way.</b><br />
<tr><td><span class="key">Artist:</span> </td><td>Richard Thomas<br /></td></tr>
</table>
</p>
<p>
<a name="card_14"><span class="cardname">Abombwe [abo]</span></a>
<span class="exp">[LoB:C/PA]</span><br />
<table><tr><td><span class="key">Cardtype:</span> </td><td>Master<br /></td></tr>
<tr><td><span class="key">Capacity:</span> </td><td>+1<br /></td></tr>
<tr><td colspan=2><b>Master: Discipline. Trifle.</b><br />
<n>Put this card on a Laibon or on a vampire with Protean [pro]. This vampire gains one level of Abombwe [abo]. Capacity increases by 1: the vampire is one generation older. Cannot be placed on a vampire with superior Abombwe.<br />
<tr><td><span class="key">Artist:</span> </td><td>Ken Meyer, Jr.<br /></td></tr>
</table>
</p>

<p>
<a name="Aeron"><span class="cardname">Aeron</span></a>
<span class="exp">[Gehenna:U]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Nosferatu antitribu<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>3<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>9<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>aus pro ANI OBF POT<br /></td></tr>
<tr><td colspan=2><b>Sabbat Archbishop of London:</b> <n>Minions opposing Aeron in combat take an additional point of damage during strike resolution if the range is close. Once each combat, Aeron may burn a blood for a press.<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Ken Meyer, Jr.<br /></td></tr>
</table>
</p>

<p>
<a name="Aire_of_Elation"><span class="cardname">Aire of Elation</span></a>
<span class="exp">[DS:C3, FN:PS3, CE:C/PTo3, Anarchs:PAB, KMW:PAn2]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Action Modifier<br /></td></tr>
<tr><td width=1><span class="key">Cost:</span> </td><td>1 blood<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>Presence<br /></td></tr>
<tr><td colspan=2><b>You cannot play another action modifier to further increase the bleed for this action.</b><br />
<n>[pre] +1 bleed; +2 bleed if the acting vampire is Toreador.<br />
<b>[PRE] +2 bleed; +3 bleed if the acting vampire is Toreador.</b><br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Greg Simanson<br /></td></tr>
</table>
</p>

<p>
<a name="Akram"><span class="cardname">Akram</span></a>
<span class="exp">[AH:V3, CE:PB]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Brujah<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>2<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>8<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>pot pre CEL QUI<br /></td></tr>
<tr><td colspan=2><b>Camarilla primogen:</b> <n>Once each turn after completing combat, if Akram and the opposing minion are still ready, Akram may burn 1 blood to begin another combat with the opposing minion.<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Terese Nielsen<br /></td></tr>
</table>
</p>

<p>
<a name="Alan_Sovereign"><span class="cardname">Alan Sovereign</span></a>
<span class="exp">[CE:V, BSC:X]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Ventrue<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>3<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>6<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>for pre AUS DOM<br /></td></tr>
<tr><td colspan=2><b>Camarilla:</b> When you play an investment card, add an additional counter to it from the blood bank.<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Steve Prescott<br /></td></tr>
</table>
</p>

<p>
<a name="Alan_Sovereign_2"><span class="cardname">Alan Sovereign</span></a>
<span class="exp">[Promo-20051001]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Ventrue<br /></td></tr>
<tr><td width=1><span class="key">Level:</span> </td><td>Advanced<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>3<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>6<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>for pre AUS DOM<br /></td></tr>
<tr><td colspan=2><b>Advanced, Camarilla:</b> While Alan is ready, you may pay some or all of the pool cost of equipping from any investment cards you control.<br />
[MERGED] During your master phase, if Alan is ready, you may move a counter from any investment card to your pool.<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Leif Jones<br /></td></tr>
</table>
</p>

<p>
<a name="Alexandra"><span class="cardname">Alexandra</span></a>
<span class="exp">[DS:V, CE:PTo]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Toreador<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>2<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>11<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>dom ANI AUS CEL PRE<br /></td></tr>
<tr><td colspan=2><b>Camarilla Inner Circle:</b> <n>Once during your turn, you may tap or untap another ready Toreador. +2 bleed.<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Lawrence Snelly<br /></td></tr>
</table>
</p>

<p>
<a name="Alfred_Benezri"><span class="cardname">Alfred Benezri</span></a>
<span class="exp">[CE:V]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Pander<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>3<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>6<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>aus dom PRE THA<br /></td></tr>
<tr><td colspan=2><b>Sabbat bishop:</b> <n>Alfred gets -1 strength in combat with an ally.<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Quinton Hoover<br /></td></tr>
</table>
</p>

<p>
<a name="Ambrogino_Giovanni"><span class="cardname">Ambrogino Giovanni</span></a>
<span class="exp">[FN:U2]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Giovanni<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>2<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>9<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>aus DOM NEC POT THA<br /></td></tr>
<tr><td colspan=2><b>Independent:</b> <n>Ambrogino has 1 vote. +1 bleed.<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Christopher Shy<br /></td></tr>
</table>
</p>
<p>
<a name="Amisa"><span class="cardname">Amisa</span></a>
<span class="exp">[AH:V3, FN:PS]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Follower of Set<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>2<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>8<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>pre pro OBF SER<br /></td></tr>
<tr><td colspan=2><b>Independent:</b> <n>Amisa has 2 votes. Amisa can tap a vampire with a capacity above 7 as a (D) action.<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Pete Venters<br /></td></tr>
</table>
</p>
<p>
<a name="Angelica_The_Canonicus"><span class="cardname">Angelica, The Canonicus</span></a>
<span class="exp">[Sabbat:V, SW:PL]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Lasombra<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>2<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>10<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>cel obf DOM OBT POT<br /></td></tr>
<tr><td colspan=2><b>Sabbat cardinal:</b> <n>Once each <span class="errata">{action that}</span> Angelica attempts to block, you may burn X master cards from your hand to give her +X intercept.<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>John Bolton<br /></td></tr>
</table>
</p>

<p>
<a name="Anna_Dictatrix11_Suljic"><span class="cardname">Anna "Dictatrix11" Suljic</span></a>
<span class="exp">[NoR:U]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Imbued<br /></td></tr>
<tr><td width=1><span class="key">Creed:</span> </td><td>Martyr<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>4<br /></td></tr>
<tr><td width=1><span class="key">Life:</span> </td><td>6<br /></td></tr>
<tr><td width=1><span class="key">Virtue:</span> </td><td>mar red vis<br /></td></tr>
<tr><td colspan=2><n>Anna may move 2 blood from the blood bank to any vampire as a +1 stealth action. During your untap phase, you may look at the top three cards of your library.<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Thomas Manning<br /></td></tr>
</table>
</p>

<p>
<a name="Anson"><span class="cardname">Anson</span></a>
<span class="exp">[Jyhad:V, VTES:V, Tenth:A]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Toreador<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>1<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>8<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>aus dom CEL PRE<br /></td></tr>
<tr><td colspan=2><b>Camarilla Prince of Seattle:</b> <n>If Anson is ready during your master phase, you get two master phase actions (instead of one).<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Mark Tedin<br /></td></tr>
</table>
</p>

<p>
<a name="Anastasz_di_Zagreb"><span class="cardname">Anastasz di Zagreb</span></a>
<span class="exp">[CE:V, KMW:PAl]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Tremere<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>3<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>8<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>ani cel dom AUS THA<br /></td></tr>
<tr><td colspan=2><b>Camarilla Tremere Justicar:</b> <n>If there are any other justicars ready, Anastasz gets 1 fewer vote from his justicar title. Anastasz may steal 1 blood as a ranged strike.<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Christopher Shy<br /></td></tr>
</table>
</p>

<p>
<a name="Bronwen"><span class="cardname">Bronwen</span></a>
<span class="exp">[Sabbat:V, SW:PB]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Brujah antitribu<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>2<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>10<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>dom obt CEL POT PRE<br /></td></tr>
<tr><td colspan=2><b>Sabbat priscus:</b> <n>Once each combat, Bronwen may dodge as a strike.<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Ken Meyer, Jr.<br /></td></tr>
</table>
</p>

<p>
<a name="Cedric"><span class="cardname">Cedric</span></a>
<span class="exp">[LoB:U]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Gargoyle<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>4<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>6<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>obf pot vis FOR<br /></td></tr>
<tr><td colspan=2><b>Camarilla. Tremere slave:</b> <n>If Cedric successfully blocks a (D) action, he may burn 1 blood when the action ends (after combat, if any) to untap. Flight [FLIGHT].<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>David Day<br /></td></tr>
</table>
</p>

<p>
<a name="Cesewayo"><span class="cardname">Cesewayo</span></a>
<span class="exp">[LoB:PO2]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Osebo<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>4<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>10<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>ani AUS CEL DOM POT THA<br /></td></tr>
<tr><td colspan=2><b>Laibon magaji:</b> <n>Once each action, Cesewayo may burn 1 blood to get +1 intercept.<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Abrar Ajmal<br /></td></tr>
</table>
</p>

<p>
<a name="Earl_Shaka74_Deams"><span class="cardname">Earl "Shaka74" Deams</span></a>
<span class="exp">[NoR:U]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Imbued<br /></td></tr>
<tr><td width=1><span class="key">Creed:</span> </td><td>Visionary<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>4<br /></td></tr>
<tr><td width=1><span class="key">Life:</span> </td><td>6<br /></td></tr>
<tr><td width=1><span class="key">Virtue:</span> </td><td>jud mar vis<br /></td></tr>
<tr><td colspan=2><n>Earl gets +1 stealth on actions other than actions to enter combat. During your untap phase, if you control more than two ready imbued, Earl burns 1 conviction [1 CONVICTION].<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>David Day<br /></td></tr>
</table>
</p>

<p>
<a name="Gracis_Nostinus"><span class="cardname">Gracis Nostinus</span></a>
<span class="exp">[CE:V]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Ventrue<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>3<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>7<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>aus for DOM PRE<br /></td></tr>
<tr><td colspan=2><b>Camarilla Primogen:</b> <n>If a younger vampire attempts to block Gracis and fails, that vampire is tapped at the end of the action.<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Max Shade Fellwalker<br /></td></tr>
</table>
</p>

<p>
<a name="Inez_Nurse216_Villagrande"><span class="cardname">Inez "Nurse216" Villagrande</span></a>
<span class="exp">[NoR:U]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Imbued<br /></td></tr>
<tr><td width=1><span class="key">Creed:</span> </td><td>Innocent<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>4<br /></td></tr>
<tr><td width=1><span class="key">Life:</span> </td><td>3<br /></td></tr>
<tr><td width=1><span class="key">Virtue:</span> </td><td>inn<br /></td></tr>
<tr><td colspan=2><n>When Inez enters play, you may search your library (shuffle afterward) or hand for a power that requires innocence and put it on her.<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Jim Pavelec<br /></td></tr>
</table>
</p>

<p>
<a name="Kabede_Maru"><span class="cardname">Kabede Maru</span></a>
<span class="exp">[LotN:U]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Assamite<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>5<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>9<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>abo pot AUS CEL OBF QUI<br /></td></tr>
<tr><td colspan=2><b>Laibon magaji:</b> <n>Kabede gets +1 intercept against political actions. (The blood curse does not affect Kabede.)<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Ken Meyer, Jr.<br /></td></tr>
</table>
</p>

<p>
<a name="Kemintiri_2"><span class="cardname">Kemintiri</span></a>
<span class="exp">[KMW:U]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Follower of Set<br /></td></tr>
<tr><td width=1><span class="key">Level:</span> </td><td>Advanced<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>2<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>10<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>aus dom OBF PRE SER THA<br /></td></tr>
<tr><td colspan=2><b>Advanced, Independent. Red List:</b> <n>+1 stealth.<br /> <n>[MERGED] Kemintiri has 3 votes (titled). She can play <span class="clarification">&lt;minion&gt;</span> cards that require Camarilla, Ventrue, and/or a justicar title <span class="clarification">&lt;as if she met that/those requirement(s)&gt;</span>.<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Lawrence Snelly<br /></td></tr>
</table>
</p>
<p>
<a name="card_15"><span class="cardname">Lázár Dobrescu</span></a>
<span class="exp">[AH:V3, FN:PR]</span><br />
<table><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td><span class="key">Clan:</span> </td><td>Ravnos<br /></td></tr>
<tr><td><span class="key">Group:</span> </td><td>2<br /></td></tr>
<tr><td><span class="key">Capacity:</span> </td><td>3<br /></td></tr>
<tr><td><span class="key">Discipline:</span> </td><td>for<br /></td></tr>
<tr><td colspan=2><b>Independent:</b> <n>Lázár may move one blood from <span class="clarification">&lt;an uncontrolled minion&gt;</span> in your prey's uncontrolled region to a vampire in your uncontrolled region as a (D) action.<br />
<tr><td><span class="key">Artist:</span> </td><td>Rebecca Guay<br /></td></tr>
</table>
</p>

<p>
<a name="Path_of_Blood"><span class="cardname">The Path of Blood</span></a>
<span class="exp">[AH:C2, FN:PA, LotN:PA2]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Master<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Assamite<br /></td></tr>
<tr><td width=1><span class="key">Cost:</span> </td><td>1 pool<br /></td></tr>
<tr><td colspan=2><b>Unique master.</b><br />
Put this card in play. Cards that require Quietus [qui] <span class="clarification">&lt;cost Assamites 1 less blood&gt;</span>. Any minion may burn this card as a (D) action; if that minion is a vampire, he or she then takes 1 unpreventable damage when this card is burned.<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Drew Tucker; Jeff Holt<br /></td></tr>
</table>
</p>

<p>
<a name="Predators_Communion"><span class="cardname">Predator's Communion</span></a>
<span class="exp">[LoB:C]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Reaction<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>Abombwe<br /></td></tr>
<tr><td colspan=2><n>[abo] [REFLEX] Cancel a frenzy card played on this vampire as it is played.<br />
<n>[abo] +1 intercept. Only usable when a vampire is acting.<br />
<b>[ABO] Only usable by a tapped vampire when a vampire is acting. This reacting vampire untaps.</b><br />
<tr><td width=1><span class="key">Artist:</span> </td><td>David Day<br /></td></tr>
</table>
</p>

<p>
<a name="Sha_Ennu"><span class="cardname">Sha-Ennu</span></a>
<span class="exp">[Third:V]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Tzimisce<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>4<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>11<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>obf tha ANI AUS CHI VIC<br /></td></tr>
<tr><td colspan=2><b>Sabbat regent:</b> <n>Vampires with capacity less than 4 cannot block Sha-Ennu. +2 bleed.<br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Richard Thomas<br /></td></tr>
</table>
</p>

<p>
<a name="Yvette_The_Hopeless"><span class="cardname">Yvette, The Hopeless</span></a>
<span class="exp">[CE:V/PTo]</span><br />
<table border=0 cellpadding=0 cellspacing=0><tr><td width=1><span class="key">Cardtype:</span> </td><td>Vampire<br /></td></tr>
<tr><td width=1><span class="key">Clan:</span> </td><td>Toreador<br /></td></tr>
<tr><td width=1><span class="key">Group:</span> </td><td>3<br /></td></tr>
<tr><td width=1><span class="key">Capacity:</span> </td><td>3<br /></td></tr>
<tr><td width=1><span class="key">Discipline:</span> </td><td>aus cel<br /></td></tr>
<tr><td colspan=2><b>Camarilla.</b><br />
<tr><td width=1><span class="key">Artist:</span> </td><td>Leif Jones<br /></td></tr>
</table>
</p>

</td></tr>
</table></body></html>
"""

TEST_RULINGS = """
<html>
<head><title>Sutekh Test Rulings</title></head>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
<body>

<p><b>AK-47:</b>
<li><span class="errata">The AK-47 provides the bearer one optional maneuver "each combat".</span>
<a href="http://groups-beta.google.com/groups?selm=46FCEF4C.6050100@white-wolf.com" target="_top" onMouseOver="window.status='Article Archived on Google';return true">[LSJ&nbsp;20070928]</a> </li>
</a>
</p>

<p><b>Ablative Skin:</b>
<li><span class="ruling">Cannot be used to prevent damage that cannot be prevented by cards that require Fortitude (e.g., Blood Rage and Blood Fury).</span>
<a href="http://groups-beta.google.com/groups?selm=36C98715.81740A4B@wizards.com" target="_top" onMouseOver="window.status='Article Archived on Google';return true">[LSJ&nbsp;19990216]</a> </li>
</a>
</p>

<p><b>Lazar Dobrescu:</b>
<li><span class="ruling">Cannot use his special ability if his controller has no vampires in her uncontrolled region.</span>
<a href="http://groups-beta.google.com/groups?selm=36C82B01.3705D2F0@wizards.com" target="_top" onMouseOver="window.status='Article Archived on Google';return true">[LSJ&nbsp;19990215]</a> </li>
</a>
</p>

</body></html>
"""



# JOLDeckParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# ELDB Deck File Parser
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Parser for JOL deck format"""

# pylint: disable-msg=W0402
# string.digits is OK
import re

class JOLDeckParser(object):
    """Parser for the JOL Deck format."""

    oCardLineRegexp = re.compile('((?P<num>[0-9]+)(\s)*x(\s)*)?(?P<name>.*)$')

    def __init__(self, oHolder):
        self._oHolder = oHolder

    def reset(self):
        """Reset the parser state"""
        pass

    def feed(self, sLine):
        """Feed the next line to the parser and extract the cards"""
        sLine = sLine.strip()
        if not sLine:
            # skip blank lines
            return
        oMatch = self.oCardLineRegexp.match(sLine)
        if oMatch is not None:
            dResults = oMatch.groupdict()
            if dResults['num']:
                iNum = int(dResults['num'])
            else:
                iNum = 1
            sName = dResults['name']
        else:
            # error condition
            raise RuntimeError('Unrecognised line for JOL format')
        # Unescape names
        if sName.endswith('(advanced)'):
            sName = sName.replace('(advanced)', '(Advanced)')
        # The following two may not be necessary, but it's not unlikely that
        # people will produce decks with these
        if sName.endswith(', The'):
            sName = 'The ' + sName[:-5]
        elif sName.endswith(', An'):
            sName = 'An ' + sName[:-4]
        # JOL has no expansion info
        self._oHolder.add(iNum, sName, None)



# test_ELDBInventoryParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test reading a card set from an ELDB inventory file"""

import unittest
from sutekh.tests.TestCore import SutekhTest, DummyHolder
from sutekh.io.ELDBInventoryParser import ELDBInventoryParser

# Needs to be a SutekhTestCase so the name mapping cache test works
class TestELDBInventoryParser(SutekhTest):
    """class for the ELDB inventory reading test"""

    sTestText1 = """
        "ELDB - Inventory"
        "Test Vamp 1",2,0,"","Crypt"
        "Lazar Dobrescu",1,0,"","Crypt"
        "Test Vamp 2",0,0,"","Crypt"
        "Test Card 1",2,0,"","Library"
        "Test Card 2",4,0,"","Library"
        "Test Card 3",12,0,"","Library"
        "Test Card 4",1,0,"","Library"
        "Test Card 5",0,0,"","Library"
        """
        # FELD creates 0,0 entries, so they need to be in the test case

    def test_basic(self):
        """Run the input test."""
        oHolder = DummyHolder()
        oParser = ELDBInventoryParser(oHolder)

        for sLine in self.sTestText1.split("\n"):
            oParser.feed(sLine + "\n")

        self.assertEqual(oHolder.name, "")

        aCards = oHolder.get_cards()

        self.assertEqual(len(aCards), 6)
        self.failUnless(("Test Vamp 1", 2) in aCards)
        self.failUnless((u"L\xe1z\xe1r Dobrescu", 1) in aCards)
        self.failUnless(("Test Card 1", 2 ) in aCards)
        self.failUnless(("Test Card 2", 4) in aCards)
        self.failUnless(("Test Card 3", 12) in aCards)
        self.failUnless(("Test Card 4", 1) in aCards)
        # We don't need to test that the 0,0 entries have been skipped, as
        # that's covered by the len test

if __name__ == "__main__":
    unittest.main()



# WwFile.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Tools for dealing with unopened files that follow the White Wolf encoding
   conventions."""

import codecs
import urllib2

# pylint: disable-msg=C0103
# These names are acceptable in this case
WW_CARDLIST_URL = [
        "http://www.white-wolf.com/vtes/index.php?line=cardlist__",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_A",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_B",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_C",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_D",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_E",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_F",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_G",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_H",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_I",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_J",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_K",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_L",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_M",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_N",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_O",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_P",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_Q",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_R",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_S",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_T",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_U",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_V",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_W",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_X",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_Y",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_Z",
        ]

WW_RULINGS_URL = "http://www.white-wolf.com/vtes/index.php?line=rulings"
# pylint: enable-msg=C0103

class WwFile(object):
    """WwFile is a convenience class which has an .open(..) method which
       returns a file-like object with the encoding set correctly.
       """

    # pylint: disable-msg=C0103
    # we accept sfFile here
    def __init__(self, sfFile, bUrl=False, bFileObj=False):
        self.sfFile = sfFile
        self.bUrl = bUrl
        self.bFileObj = bFileObj

        if bUrl and bFileObj:
            raise ValueError("WwFile cannot be both a URL and a fileobject")

    # pylint: enable-msg=C0103

    def open(self):
        """Return a file object for the file."""
        if self.bFileObj:
            return self.sfFile
        elif self.bUrl:
            return codecs.EncodedFile(urllib2.urlopen(self.sfFile), 'utf8',
                    'cp1252')
        else:
            return codecs.open(self.sfFile, 'rU', 'cp1252')




# PhysicalCardParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2005, 2006, 2007 Simon Cross <hodgestar@gmail.com>
# Copyright 2007, 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Read physical cards from an XML file which looks like:

   <cards sutekh_xml_version="1.0">
     <card id='3' name='Some Card' count='5' expansion="Some Expansion" />
     <card id='3' name='Some Card' count='2'
        Expansion="Some Other Expansion" />
     <card id='5' name='Some Other Card' count='2'
       expansion="Some Expansion" />
   </cards>
   into the default PhysicalCardSet 'My Collection'.
   """

from sutekh.core.CardSetHolder import CachedCardSetHolder
from sutekh.core.CardLookup import DEFAULT_LOOKUP
from sqlobject import sqlhub
# pylint: disable-msg=E0611, F0401
# xml.etree is a python2.5 thing
try:
    from xml.etree.ElementTree import parse, fromstring, ElementTree
except ImportError:
    from elementtree.ElementTree import parse, fromstring, ElementTree
# pylint: enable-msg=E0611, F0401

class PhysicalCardParser(object):
    """Implement the PhysicalCard Parser.

       We read the xml file into a ElementTree, then walk the tree to
       extract the cards.
       """
    aSupportedVersions = ['1.0', '0.0']

    def __init__(self):
        self.oCS = None
        self.oTree = None

    def _convert_tree(self):
        """parse the elementtree into a card set holder"""
        self.oCS = CachedCardSetHolder()
        self.oCS.name = "My Collection"
        oRoot = self.oTree.getroot()
        if oRoot.tag != 'cards':
            raise RuntimeError("Not a Physical card list")
        if oRoot.attrib['sutekh_xml_version'] not in self.aSupportedVersions:
            raise RuntimeError("Unrecognised XML File version")
        for oElem in oRoot:
            if oElem.tag == 'card':
                iCount = int(oElem.attrib['count'], 10)
                sName = oElem.attrib['name']
                try:
                    sExpansionName = oElem.attrib['expansion']
                    if sExpansionName == "None Specified":
                        sExpansionName = None
                except KeyError:
                    sExpansionName = None
                self.oCS.add(iCount, sName, sExpansionName)

    def _commit_tree(self, oCardLookup, dLookupCache):
        """Commit the tree to the database.

           We use the card set holder, so it calls the appropriate card lookup
           function for unknown cards
           """
        oOldConn = sqlhub.processConnection
        sqlhub.processConnection = oOldConn.transaction()
        self.oCS.create_pcs(oCardLookup, dLookupCache)
        sqlhub.processConnection.commit(close=True)
        sqlhub.processConnection = oOldConn
        self.oCS = None

    # pylint: disable-msg=W0102
    # W0102 - {} is the right thing here
    def parse(self, fIn, oCardLookup=DEFAULT_LOOKUP, bIgnoreWarnings=True,
            dLookupCache={}):
        """Read the file object fIn into the database."""
        self.oTree = parse(fIn)
        self._convert_tree()
        self._commit_tree(oCardLookup, dLookupCache)
        if not bIgnoreWarnings:
            return self.oCS.get_warnings()
        else:
            return []

    def parse_string(self, sIn, oCardLookup=DEFAULT_LOOKUP,
            bIgnoreWarnings=True, dLookupCache={}):
        """Read the string sIn into the database."""
        self.oTree = ElementTree(fromstring(sIn))
        self._convert_tree()
        self._commit_tree(oCardLookup, dLookupCache)
        if not bIgnoreWarnings:
            return self.oCS.get_warnings()
        else:
            return []



# test_SLInventoryParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2009 Simon Cross hodgestar+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test reading a card set from an Secret Libary inventory file"""

import unittest
from sutekh.tests.TestCore import SutekhTest, DummyHolder
from sutekh.io.SLInventoryParser import SLInventoryParser

class TestSLInventoryParser(SutekhTest):
    """Class for the Secret Library inventory file parser tests"""

    sTestText1 = """
        ***SL***CRYPT***
        2;2;Abebe
        2;2;Alan Sovereign (Adv)
        4;3;Fran\xc3\xa7ois Warden Loehr
        ***SL***LIBRARY***
        1;1;Absimiliard's Army
        5;5;Ahriman's Demesne
        4;4;Atonement
        1;1;Textbook Damnation, The
        8;8;Watch Commander
        ***SL***ENDEXPORT***
        """

    def test_basic(self):
        """Run the Secret Libary inventory input test."""
        oHolder = DummyHolder()
        oParser = SLInventoryParser(oHolder)

        for sLine in self.sTestText1.split("\n"):
            oParser.feed(sLine + "\n")

        aCards = oHolder.get_cards()

        self.assertEqual(len(aCards), 8)
        self.failUnless(("Abebe", 2) in aCards)
        self.failUnless(("Alan Sovereign (Advanced)", 2) in aCards)
        self.failUnless(("Fran\xc3\xa7ois Warden Loehr", 4) in aCards)
        self.failUnless(("Absimiliard's Army", 1) in aCards)
        self.failUnless(("Ahriman's Demesne", 5) in aCards)
        self.failUnless(("Atonement", 4) in aCards)
        self.failUnless(("The Textbook Damnation", 1) in aCards)
        self.failUnless(("Watch Commander", 8) in aCards)

if __name__ == "__main__":
    unittest.main()



# CardTextView.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2005, 2006, 2007 Simon Cross <hodgestar@gmail.com>
# Copyright 2006 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Widget for displaying the card text for the given card."""

import gtk
import pango

class CardTextBuffer(gtk.TextBuffer, object):
    """Buffer object which holds the actual card text.

       This is also responsible for nicely formatting the output.
       """
    # pylint: disable-msg=R0904
    # gtk.Widget, so many public methods
    def __init__(self):
        super(CardTextBuffer, self).__init__(None)

        # See http://www.pygtk.org/pygtk2reference/class-gtktexttag.html
        # for some possible properties

        oIconTabs = pango.TabArray(1, True)
        oIconTabs.set_tab(0, pango.TAB_LEFT, 15) # Icons are scaled to 15'ish

        self.create_tag("label", underline=pango.UNDERLINE_SINGLE)

        self.create_tag("card_name", weight=pango.WEIGHT_BOLD)
        self.create_tag("cost", style=pango.STYLE_ITALIC)
        self.create_tag("life", style=pango.STYLE_ITALIC)
        self.create_tag("capacity", style=pango.STYLE_ITALIC)
        self.create_tag("group", style=pango.STYLE_ITALIC)
        self.create_tag("level", style=pango.STYLE_ITALIC)
        self.create_tag("burn_option", style=pango.STYLE_ITALIC)
        self.create_tag("card_type", style=pango.STYLE_ITALIC, left_margin=15,
                tabs=oIconTabs)
        self.create_tag("clan", style=pango.STYLE_ITALIC, left_margin=15,
                tabs=oIconTabs)
        self.create_tag("sect", style=pango.STYLE_ITALIC)
        self.create_tag("title", style=pango.STYLE_ITALIC)
        self.create_tag("creed", style=pango.STYLE_ITALIC, left_margin=15,
                tabs=oIconTabs)
        self.create_tag("discipline", style=pango.STYLE_ITALIC, left_margin=15,
                tabs=oIconTabs)
        self.create_tag("virtue", style=pango.STYLE_ITALIC, left_margin=15,
                tabs=oIconTabs)
        self.create_tag("expansion", style=pango.STYLE_ITALIC, left_margin=15,
                tabs=oIconTabs)
        self.create_tag("ruling", left_margin=15,
                tabs=oIconTabs)
        self.create_tag("card_text", style=pango.STYLE_ITALIC)
        self._oIter = None

    # pylint: disable-msg=W0142
    # ** magic OK here
    def tag_text(self, *aArgs, **kwargs):
        """Inset the text (possibly with tags) at the current position"""
        self.insert_with_tags_by_name(self._oIter, *aArgs, **kwargs)

    # pylint: enable-msg=W0142

    def labelled_value(self, sLabel, sValue, sTag, oIcon=None):
        """Add a single value to the buffer."""
        self.tag_text("\n")
        self.tag_text(sLabel, "label")
        self.tag_text(": ")
        if oIcon:
            self.insert_pixbuf(self._oIter, oIcon)
            self.insert(self._oIter, ' ')
        self.tag_text(sValue, sTag)

    def labelled_list(self, sLabel, aValues, sTag, dIcons=None):
        """Add a list of values to the Buffer"""
        def _insert_line(sText, sTag, oPixbuf):
            """Insert a line, prefixed by oPixbuf."""
            if oPixbuf:
                self.tag_text('\n', sTag)
                # This is a bit complicated - we need to insert the pixbuf
                # into a region already tagged with the correct tag for
                # the margins + tabs to work. We use a text mark to note
                # where to insert the pixbuf, insert the text, then
                # we backtrack to insert the pixbuf.
                oMark1 = self.create_mark(None, self._oIter, True)
                self.tag_text('\t%s' % sText, sTag)
                oMarkEnd = self.create_mark(None, self._oIter, True)
                self.insert_pixbuf(self.get_iter_at_mark(oMark1), oPixbuf)
                # We've invalidated self._oIter, so restore it
                self._oIter = self.get_iter_at_mark(oMarkEnd)
            else:
                self.tag_text('\n*\t%s' % sText, sTag)
        self.tag_text("\n")
        self.tag_text(sLabel, "label")
        self.tag_text(":")
        for sValue in aValues:
            if dIcons:
                if dIcons.has_key(sValue) and dIcons[sValue]:
                    _insert_line(sValue, sTag, dIcons[sValue])
                elif dIcons.has_key(sValue.lower()) and dIcons[sValue.lower()]:
                    _insert_line(sValue, sTag, dIcons[sValue.lower()])
                else:
                    _insert_line(sValue, sTag, None)
            else:
                _insert_line(sValue, sTag, None)

    def labelled_compact_list(self, sLabel, aValues, sTag):
        """More compact list for clans, etc."""
        self.labelled_value(sLabel, " / ".join(aValues), sTag)

    def labelled_exp_list(self, sLabel, dValues, sTag):
        """Special case for expansion labels"""
        aValues = []
        for sExp in sorted(dValues):
            aRarities = list(set(dValues[sExp]))
            # Use set to avoid Precon, Precon and other duplicates
            sValue = '%s (%s)' % (sExp, ", ".join(sorted(aRarities)))
            # sort for consistent ordering
            aValues.append(sValue)
        self.labelled_list(sLabel, aValues, sTag)

    def reset_iter(self):
        """Reset the iterator to point at the start of the buffer."""
        self._oIter = self.get_iter_at_offset(0)


class CardTextView(gtk.TextView, object):
    """TextView widget which holds the TextBuffer.

       This handles extracting the text from the database for the card,
       and feeding it to the buffer in suitable chunks.
       """
    # pylint: disable-msg=R0904
    # gtk.Widget, so many public methods
    def __init__(self, oController, oIconManager, bVerbose):
        super(CardTextView, self).__init__()
        # Can be styled as frame_name.view
        self.__oController = oController
        self.__oBuf = CardTextBuffer()

        self.set_buffer(self.__oBuf)
        self.set_editable(False)
        self.set_cursor_visible(False)
        self.set_wrap_mode(gtk.WRAP_WORD)
        self._oIconManager = oIconManager
        if bVerbose:
            oContext = self.get_pango_context()
            print 'Pango Language : ', oContext.get_language()
            print 'Pango Font Description : ', oContext.get_font_description()

    def set_card_text(self, oCard):
        """Add the text for oCard to the TextView."""
        oStart, oEnd = self.__oBuf.get_bounds()
        self.__oBuf.delete(oStart, oEnd)
        self.print_card_to_buffer(oCard)

    # pylint: disable-msg=R0912, R0915
    # We need to consider all cases for oCard, so need the branches
    # and statements
    def print_card_to_buffer(self, oCard):
        """Format the text for the card and add it to the buffer."""
        self.__oBuf.reset_iter()

        self.__oBuf.tag_text(oCard.name, "card_name")

        if not oCard.cost is None:
            if oCard.cost == -1:
                sCost = "X " + str(oCard.costtype)
            else:
                sCost = str(oCard.cost) + " " + str(oCard.costtype)
            self.__oBuf.labelled_value("Cost", sCost, "cost")

        if not oCard.capacity is None:
            self.__oBuf.labelled_value("Capacity", str(oCard.capacity),
                    "capacity")

        if not oCard.life is None:
            self.__oBuf.labelled_value("Life", str(oCard.life), "life")

        if not oCard.group is None:
            if oCard.group == -1:
                sGroup = 'Any'
            else:
                sGroup = str(oCard.group)
            self.__oBuf.labelled_value("Group", sGroup, "group")

        if not oCard.level is None:
            oIcon = self._oIconManager.get_icon_by_name('advanced')
            self.__oBuf.labelled_value("Level", str(oCard.level), "level",
                    oIcon)

        if len(oCard.cardtype) == 0:
            aTypes = ["Unknown"]
        else:
            aTypes = [oT.name for oT in oCard.cardtype]
        dIcons = self._oIconManager.get_icon_list(oCard.cardtype)
        self.__oBuf.labelled_list("Card Type", aTypes, "card_type", dIcons)

        if oCard.burnoption:
            oIcon = self._oIconManager.get_icon_by_name('burn option')
            self.__oBuf.labelled_value("Burn Option:", "Yes", "burn_option",
                    oIcon)

        if not len(oCard.clan) == 0:
            dIcons = self._oIconManager.get_icon_list(oCard.clan)
            self.__oBuf.labelled_list("Clan",
                    [oC.name for oC in oCard.clan], "clan", dIcons)

        if not len(oCard.creed) == 0:
            dIcons = self._oIconManager.get_icon_list(oCard.creed)
            self.__oBuf.labelled_list("Creed",
                    [oC.name for oC in oCard.creed], "creed", dIcons)

        if not len(oCard.sect) == 0:
            self.__oBuf.labelled_compact_list("Sect",
                    [oC.name for oC in oCard.sect], "sect")

        if not len(oCard.title) == 0:
            self.__oBuf.labelled_compact_list("Title",
                    [oC.name for oC in oCard.title], "title")

        if not len(oCard.discipline) == 0:
            aDis = []
            aDis.extend(sorted([oP.discipline.name.upper() for oP in
                oCard.discipline if oP.level == 'superior']))
            aDis.extend(sorted([oP.discipline.name for oP in oCard.discipline
                if oP.level != 'superior']))
            dIcons = self._oIconManager.get_icon_list(oCard.discipline)
            self.__oBuf.labelled_list("Disciplines", aDis, "discipline",
                    dIcons)

        if not len(oCard.virtue) == 0:
            dIcons = self._oIconManager.get_icon_list(oCard.virtue)
            self.__oBuf.labelled_list("Virtue",
                    [oC.name for oC in oCard.virtue], "virtue", dIcons)

        if not len(oCard.rarity) == 0:
            dExp = {}
            for oPair in oCard.rarity:
                dExp.setdefault(oPair.expansion.name, [])
                dExp[oPair.expansion.name].append(oPair.rarity.name)
            self.__oBuf.labelled_exp_list("Expansions", dExp, "expansion")

        if not len(oCard.rulings) == 0:
            aRulings = [oR.text.replace("\n", " ") + " " + oR.code for oR
                    in oCard.rulings]
            self.__oBuf.labelled_list("Rulings", aRulings, "ruling")

        self.__oBuf.tag_text("\n\n")
        self.__oBuf.tag_text(oCard.text.replace("\n", " "),
                "card_text")
